      subroutine couplsmb(e,sw2,mw,mh)
c----------------------------------------------------------------------c
c Returns the standard model couplings
c 
c Input: e - electric charge,
c        sw2 - complex electroweak mixing parameter,
c        mw - complex W boson mass
c        mh - complex Higgs boson mass
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 18 June, 2001; modified on 20 September, 2001
c----------------------------------------------------------------------c

      use parproc

      implicit none

      real(8) :: e
      complex(8) :: sw2,mw,mh

      complex(8) :: sw,cw2,cw
      complex(8), parameter :: ocom=(0.d0,0.d0)

      sw=sqrt(sw2)
      cw2=1.d0-sw2
      cw=sqrt(cw2)

c Photon couplings to fermions:

      gnn=ocom
      gll=cmplx(-e,0.d0,8)
      guu=cmplx(2.d0/3.d0*e,0.d0,8)
      gdd=cmplx(-1.d0/3.d0*e,0.d0,8)

c Z boson couplings to fermions:

      znn=(/ocom,1.d0/(2.d0*sw*cw)*e/)
      zll=(/sw/cw*e,-(1.d0-2.d0*sw2)/(2.d0*sw*cw)*e/)
      zuu=(/-2.d0/3.d0*sw/cw*e,
     &                  (1.d0-4.d0/3.d0*sw2)/(2.d0*sw*cw)*e/)
      zdd=(/sw/cw/3.d0*e,-(1.d0-2.d0/3.d0*sw2)/(2.d0*sw*cw)*e/)

c W boson coupling to fermions:

      wff=(/ocom,e/sqrt(2.d0)/sw/)

c Triple gauge boson couplings:

      gww=cmplx(-e,0.d0,8)
      zww=-cw/sw*e

c Quartic gauge boson couplings:

      ggww=cmplx(e**2,0.d0,8)
      gzww=cw/sw*e**2
      zzww=cw2/sw2*e**2
      wwww=-1.d0/sw2*e**2

c Higgs boson couplings to fermions of a given process:

      if (ischeme == 1) then
       hff=-e/(2.d0*sw*mw)*(/cm1,cm2,cm3,cm4,cm5,cm6,cm7,cm8/)
      else
       hff=-e/(2.d0*sw*mw)*cmplx((/m1,m2,m3,m4,m5,m6,m7,m8/),0.d0,8)
      end if

c Higgs boson couplings to Z and W bosons:

      hzz=e*mw/sw/cw2
      hww=e*mw/sw

c Scalar-fermion couplings:

      csff=e/(sqrt(2.d0)*sw*mw)
      c3ff=cmplx(0.d0,-e,8)/(2.d0*sw*mw)

c Scalar-gauge boson couplings:

      csaw=e*mw
      cszw=-e*mw*sw/cw 

c Outgoing phi-(p1), incoming phi-(p2):

      cass=-e
      czss=e*(sw**2-cw**2)/(2.d0*sw*cw)

c Outgoing phi3(p1), incoming phi-(p2):

      cw3s=cmplx(0.d0,e,8)/(2.d0*sw)

c Outgoing H(p1), incoming phi-(p2):

      cwhs=-e/(2.d0*sw)

c Outgoing phi3(p1), incoming H(p2):

      cz3h=cmplx(0.d0,e,8)/(2.d0*sw*cw)

c Triple scalar couplings:

      chss=-e*mh**2/(2.d0*sw*mw)

      end


