c----------------------------------------------------------------------c
c The unpolarized cross section of 
c
c e+(p1) + e-(p2) -> b(p3) + f1u(p4) + f1d~(p5) + b~(p6) + f2d(p7) + f2u~(p8)
c 
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 15 June, 2001; modified 13 March, 2002
c----------------------------------------------------------------------c

      use drivec 
      use inprms   ! Defines initial parameters
      use mathprms ! Defines mathematical constants
      use parproc  ! Parameters for a given process
      use kincuts
      use distribs ! Defines parameters of distributions

      implicit none

      integer, parameter :: ne= 1  ! No of the cms energies

      real(8), dimension(ne) :: ! The cms energies:
     &            aecm=(/500.d0/)
c     &            aecm=(/340.d0,350.d0,360.d0,370.d0,380.d0,
c     &                   390.d0,400.d0,450.d0,500.d0/)
c     &     aecm=(/310.d0,320.d0,330.d0,340.d0,342.5d0,345.d0,347.5d0,
c     &            350.d0,355.d0,360.d0,365.d0,370.d0,
c     &            375.d0,380.d0,385.d0,390.d0,395.d0,400.d0,405.d0,
c     &            407.5d0,410.d0,412.5d0,415.d0,420.d0,430.d0,440.d0,
c     &            460.d0,480.d0,500.d0,550.d0,575.d0,
c     &            600.d0,650.d0,700.d0,750.d0,800.d0,900.d0,950.d0,
c     &            1.d3,1.1d3,1.2d3,1.3d3,1.4d3,1.6d3,1.8d3,2.d3/)

      integer :: ndim,ncall

      integer :: n,ncall0,j,ie,id,neff0,ncalla,itmxa,neffa,iwwa,
     &           itopa,ncallww,itmxww,neffww


      real(8) :: cross,cs0,sd0,chi20,cssb,sdsb,chi2sb,cshb,sdhb,chi2hb,
     &           cs0i,sd0i,chi20i,csta,sdta,chi2ta,theb,sdwwa,chi2wwa,
     &           cswwi,sdwwi,chi2wwi,cswwa

      integer :: i,iscan0,nscan0,iscanww,nscanww,nscan
      real(8) :: a
      real(8) :: stl,stu
 
      real(8), dimension(ne) :: acs0,asd0,awwa,asda,cstta,sdtta
      real(8) :: cssc(nkin),cswwsc(nkina),arg,val

      external cross,crosstopa


c Choose a decay mode of W+ and W-: 1=leptonic, 2=hadronic, 
c e.g. W+ -> vl l+ (iwp=1) or W+ -> u d~ (iwp=2):
c and specify the fermion family index: ifp,ifm = 1,2,3

      iwp=1
      ifp=2
      iwm=2
      ifm=1

c Calculate the Born cross section with 'ncall0' calls utilizing 
c the 6 particle kinematics: iborn = 1(yes)/else(no)?
c Generate the unweighted events: imc=1(yes)/else(no)?

      iborn=1
      ncall0=20000
      imc=0

c Scan the Born cross section with `nscan0' calls: iscan0 = 1(yes)/else(no)?

      iscan0=1
      nscan0=200

c Calculate an approximate cross section of e+ e- -> b W+ b~ W- -> 6f:
c iwwa=1 (yes)/ else (no:

      iwwa=1
      ncallww=20000

c Scan the Born cross section in the narrow W-width approxmation with 
c `nscanww' calls: iscanww = 1(yes)/else(no)?

      iscanww=1
      nscanww=200

c Calculate an approximate cross section of e+ e- -> t t~ -> 6f:
c itopa=1 (yes)/ else (no:

      itopa=1
      ncalla=20000

c Include the Higgs boson exchange: ihiggs = 1 (yes) / else (no)?

      ihiggs=1

c Calculate the electroweak contributions only, iew =1 (yes)/else (ew
c + QCD diagrams):

      iew=0

c Calculate the top-antitop resonant diagrams only, itt =1 (yes)/else (all 
c the diagrams):

      itt=0

c Calculate the double W resonant diagrams only, iww =1 (yes)/else (all 
c the diagrams):

      iww=0

c Choose the scheme:
c     ischeme = 1 - complex mass scheme/ else - fixed width scheme.

      ischeme=1

c If ischeme=1, then should alpha_W be complex (iaplw=1) or real (ialpw=0)?

      ialpw=1


c Impose the cuts, icuts= 1 (yes - canonical)/ 2 (yes - a cut on the final 
c state electron)/ 3 (yes - a cut on the final state electron and positron)
c / else (no):

      icuts=0

      if (icuts == 1) then

c The kinematical cuts:

      ctlb=cos(5.d0/180.d0*pi)
      ctqb=cos(5.d0/180.d0*pi)
      ctll=cos(5.d0/180.d0*pi)
      ctlq=cos(5.d0/180.d0*pi)
      ecutl=10.0d0
      ecutq=10.0d0
      mqq=10.d0
      mll=0.d0

      end if ! icuts

c Test against Madgraph, imad=1(yes-commented)/else(no), 
c Test the phase space volume, imad=2
c Madgraph fails in case of the mixed electroweak-QCD channels, where it 
c generates either the electroweak or QCD contributions.

      imad=0

c Calculate distributions, idis=1(yes)/0(no)?

      idis=0

      if (idis == 1) then
       stl=(mt-5.d0*gamt)**2
       stu=(mt+5.d0*gamt)**2
       xmin=(/ stl, stl, sqrt(stl),-1.d0,-1.d0,-1.d0/)
       xmax=(/ stu, stu, sqrt(stu), 1.d0, 1.d0, 1.d0/)
       nbs= (/ 160, 160, 160, 60, 60, 60/)
       do j=1,ndist
        xdi(0,j)=xmin(j)
        dx(:,j)=(xmax(j)-xmin(j))/nbs(j)
        do i=1,nbs(j)
         xdi(i,j)=xdi(i-1,j)+dx(i,j)
        end do
       end do
      end if

c Specify the maximum of the cross section (relevant if iscan0=0 or iscanww=0)

      crmax=1.d3

c----------------------------------------------------------------------c
c End of input
c----------------------------------------------------------------------c

c Determine parameters of the proces:

      call parfix

      write(*,4000)
      write(*,2000)
      write(*,5000)
      write(*,2000)
      write(6,2010) chwp,chwm
      write(*,2000)
      write(*,5010)
      write(*,2000)
      write(*,5020)
      write(*,5030)
      write(*,5040)
      write(*,5050)
      write(*,2000)
      write(*,4000)

 2000 format('#')
 2010 format('#                         e+ e- -> b ',a6,' b~ ',a6)
 5000 format('# Approaching the process:')
 5010 format('# with the program ''eett6f v. 1.0'' by')
 5020 format('#                            Karol Kolodziej')
 5030 format('#                         University of Silesia')
 5040 format('#                            Katowice, Poland')
 5050 format('#                        e-mail: kolodzie@us.edu.pl')


c The energy loop:

      do ie=1,ne

      ecm=aecm(ie)     !  The c.m.s. energy

      if (ecm < m3+mw+m6+mw) then 
       write(*,*)'The CMS energy below the 2(mW+mb) threshold'
       stop
      end if

      write(*,2000)

      if (ischeme == 1) then
        write(6,2095)
 2095   format('# Complex mass scheme')
        write(6,5029) csw2
 5029   format('# with sw2 = (',f18.15,',',f18.15,')')
        if (ialpw == 0) then
         write(6,5031) cralpw
 5031    format('# and real 1/alpha_W = (',f19.15,',',f19.15,')')
        else if (ialpw == 1) then
         write(6,5041) cralpw
 5041    format('# and complex 1/alpha_W = (',f19.15,',',f19.15,')')
        else
         write(6,5051) ialpw
 5051    format('# ialpw =',i2,' is not allowed')
        end if

      else
        write(6,2098)
 2098   format('# Fixed width scheme')
        write(6,5033) sw2
 5033   format('# with real sw2 =', f18.15)
        write(6,5035) ralpw
 5035   format('# and real 1/alpha_W = ',f19.15)
      end if

      write(*,2000)

      write(6,2120)
 2120 format('# Gauge boson masses and widths:',
     &           ' W, Gm_W, Z, Gm_Z')
      write(6,1002) mw,gamw,mz,gamz
      write(*,2000)
 1002 format('# ',4(f12.5))

      if (ihiggs == 1) then
       write(6,2122)
 2122  format('# Higgs boson mass and width: H, Gm_H')
       write(6,2002) mh,gamhig
       write(*,2000)
 2002  format('# ',2(e15.5))
      end if

      write(6,1003) ralp0,alphas,gmu
 1003 format('# 1/alpha0 = ',f11.7,'   alphas = ',f9.7,
     &       '   G_mu = ',e13.6)

      write(*,2000)
      write(6,2110)
 2110 format('# t-quark mass and width:')

      write(6,1020) mt,gamt
      write(*,2000)

      write(6,2100) chwp,chwm
 2100 format('# External fermion masses: e- b ',a6,' ',a6,':')
      write(6,1001) m1,m3,m4,m5
      write(6,1008) m7,m8
 1001 format('#',4(e15.8))
 1008 format('#',2(e15.8))
      write(*,2000)

 1020 format('#',f10.2,f12.5)

      write(*,4000)
      write(6,1006) ecm
      write(*,4000)
 1006 format('#                       E_cm = ',f11.3,' GeV')

      write(*,5070)
 5070 format('# All cross sections in femtobarns (fb)')
      write(*,4000)


      if (icuts == 1) then
        write(6,2090)
 2090   format('# KINEMATICAL CUTS IMPOSED')
        write(*,4000)
      else if (icuts == 2) then
        write(6,2091) theb,180.d0-theb
 2091   format('# A FINAL ELECTRON ANGLE ',e12.5,' < theta <',e12.5)
        write(*,4000)
      else if (icuts == 3) then
        write(6,2092) theb,180.d0-theb
 2092   format('# FINAL ELECTRON AND POSITRON ANGLE ',e12.5,
     &            ' < theta <',e12.5)
        write(*,4000)
      end if


      if (iborn == 1) then

      write(*,5060)
 5060 format('# The Born cross section with full 6 particle',
     &       ' kinematics:')
      write(*,4000)

      if (iew == 1) then
       write(6,2075)
 2075  format('# ONLY THE ELECTROWEAK FEYNMAN DIAGRAMS!')
       write(*,4000)
      end if

      if (itt == 1) then
       write(6,2080)
 2080 format('# ONLY THE TOP-ANTITOP RESONANT FEYNMAN DIAGRAMS!')
       write(*,4000)
      end if

      if (iww == 1) then
       write(6,2085)
 2085 format('# ONLY THE DOUBLE W RESONANT FEYNMAN DIAGRAMS!')
       write(*,4000)
      end if

      aw=aw0
      idrive=1
      ndim=13
      iscan=iscan0

      if (iscan == 1) then

c Scan the Born cross section in order to find the dominant channels:

        ncall=nscan0
        cs0=0.d0
        sd0=0.d0
        icall=0
        cssc=zero
        crmax=zero

        write(*,5110)
 5110 format('# Results of the initial scan:')
        write(*,5120)
 5120 format('# Kinematics        Weight          Integral        ',
     &       'St.dev.       No of calls')

        do ikin=1,nkin
         if (aw(ikin) > 0.d0) then

          call carlos(cross,ndim,ncall,cs0i,sd0i)

          write(*,3002) ikin,aw(ikin),aw(ikin)*cs0i,aw(ikin)*sd0i,ncall
 3002 format('#  ',i4,'     ',e17.8,e17.8,e17.8,i10)

          cs0=cs0+aw(ikin)*cs0i
          cssc(ikin)=aw(ikin)*cs0i
         end if
        end do

        write(*,4000)
        write(*,3800) icall,cs0
 3800 format('# Born scanned with',i8,' calls = ',e15.8)
        write(*,3810) crmax
 3810 format('# The maximum value of the cross section= ',e15.8)

        crmax=1.5d0*crmax

        write(*,3820) crmax
 3820 format('# shifted to ',e15.8)
        write(*,4000)

c Set weights for the final integration:

         neff0=0

         xmc(0)=zero

         do ikin=1,nkin

          if (aw(ikin) > zero) then
           aw(ikin)=cssc(ikin)/cs0
          else
           aw(ikin)=zero
          end if

          xmc(ikin)=xmc(ikin-1)+aw(ikin)

          if (xmc(ikin) > 0.999999999999d0) then
           xmc(ikin)=1.d0
           write(*,3456) ikin,aw(ikin),xmc(ikin)-xmc(ikin-1)
 3456 format('# Weight',i5,' = ',e23.16,' rescaled to',e23.16)
           aw(ikin)=xmc(ikin)-xmc(ikin-1)
           aw(ikin+1:)=zero
           exit
          end if

         end do

      iscan=0

      else ! iscan

c Set weights for the final integration:

      neff0=0

      xmc(0)=zero

      do ikin=1,nkin

       xmc(ikin)=xmc(ikin-1)+aw(ikin)

       if (xmc(ikin) > 0.999999999999d0) then
        xmc(ikin)=1.d0
        write(*,3456) ikin,aw(ikin),xmc(ikin)-xmc(ikin-1)
        aw(ikin)=xmc(ikin)-xmc(ikin-1)
        aw(ikin+1:)=zero
        exit
       end if

      end do


      end if ! iscan



c The Born cross section with standard deviation:

      ncall=ncall0

      cs0=0.d0
      sd0=0.d0
      icall=0

      ieff=0

      if (idis == 1) then
       ncs=0
       di=zero
      end if

      write(*,3200)
 3200 format('# Calculating the Born cross section with the weights:')

      do ikin=1,nkin
       write(*,3300) ikin,aw(ikin),xmc(ikin)
 3300 format('#',i3, 2e16.8)
      end do

      call carlos(cross,ndim,ncall,cs0,sd0)

      neff0=icall

 2987 format('# Integral =',e13.6,'  Standard dev. =',e9.2,' with ',i9,
     &                                                         ' calls')

 4000 format('#-------------------------------------------------------',
     &       '-----------------------#')

      write(*,4000)
      write(*,2989)
 2989 format('# Final result:')

      write(6,2987) cs0,sd0,neff0

      write(*,4000)
      write(*,3030) dble(ieff)/dble(icall)
 3030 format('# Efficiency of the events acceptance =',e10.3)


      if (idis == 1) then

       do j=1,ndist

       write(*,4000)
        write(6,3000) j
       write(*,4000)
 3000 format('# Distribution No. ',i3)
        arg=xmin(j)

        do n=1,nbs(j)

         if (ncs(n,j) > 0 .and. dx(n,j) > zero) then
          val=di(n,j)/dx(n,j)
         else
          val=zero
         end if

         if (j < 3) then
          write(6,1010) sqrt(xdi(n-1,j)),val
 1010 format(2e11.4)
          write(6,1010) sqrt(xdi(n,j)),val
         else
          write(6,1010) xdi(n-1,j),val
          write(6,1010) xdi(n,j),val
         end if
        end do

       end do

      write(*,4000)
      write(*,5080)
 5080 format('# Test of the distributions:')

      write(*,5085) cs0,neff0
 5085 format('# Total:  ',e15.8,i12)

      do j=1,ndist
       write(*,5090) j,sum(di(:,j)),sum(ncs(:,j))
 5090 format('# ',i8,e15.8,i12)
      end do

      end if ! idis



      else

      cs0=0.d0
      sd0=0.d0

      end if ! iborn



      if (iwwa == 1) then

      write(*,4000)
      write(*,5065)
 5065 format('# The narrow W-width',
     #       ' approximation, e+ e- -> b W+ b~ W- -> 6f:')
      write(*,4000)

      if (itt == 1) then
       write(6,2080)
       write(*,4000)
      end if


      awa=awa0
      idrive=2
      ndim=7
      iscan=iscanww


      if (iscan == 1) then

c Scan the Born cross section in the narrow W-width approximation in order 
c to find the dominant channels:

        ncall=nscanww
        cswwa=0.d0
        sdwwa=0.d0
        icall=0
        cswwsc=zero
        crmax=zero

        write(*,5110)
        write(*,5120)

        do ikin=1,nkina
         if (awa(ikin) > 0.d0) then

          call carlos(cross,ndim,ncall,cswwi,sdwwi)

          write(*,3002) ikin,awa(ikin),awa(ikin)*cswwi,awa(ikin)*sdwwi,
     &                  ncall

          cswwa=cswwa+awa(ikin)*cswwi
          cswwsc(ikin)=awa(ikin)*cswwi
         end if
        end do

        write(*,4000)
        write(*,3800) icall,cswwa
        write(*,3810) crmax

        crmax=1.5d0*crmax

        write(*,3820) crmax
        write(*,4000)

c Set weights for the final integration:

         neffww=0

         xmc(0)=zero

         do ikin=1,nkina

          if (awa(ikin) > zero) then
           awa(ikin)=cswwsc(ikin)/cswwa
          else
           awa(ikin)=zero
          end if

          xmc(ikin)=xmc(ikin-1)+awa(ikin)

          if (xmc(ikin) > 0.9999999999999d0) then
           xmc(ikin)=1.d0
           write(*,3456) ikin,awa(ikin),xmc(ikin)-xmc(ikin-1)
           awa(ikin)=xmc(ikin)-xmc(ikin-1)
           awa(ikin+1:)=zero
           exit
          end if

         end do

      iscan=0

      else ! iscan

c Set weights for the final integration:

      neffww=0

      xmc(0)=zero

      do ikin=1,nkina
       xmc(ikin)=xmc(ikin-1)+awa(ikin)
       if (xmc(ikin) > 0.9999999999999d0) then
        xmc(ikin)=1.d0
        write(*,3456) ikin,awa(ikin),xmc(ikin)-xmc(ikin-1)
        awa(ikin)=xmc(ikin)-xmc(ikin-1)
        awa(ikin+1:)=zero
        exit
       end if
      end do

      end if ! iscan



c The Born cross section in the narrow W-width approximation with standard 
c deviation:

      ncall=ncallww

      cswwa=0.d0
      sdwwa=0.d0
      icall=0

      ieff=0

      if (idis == 1) then
       ncs=0
       di=zero
      end if

      write(*,3200)

      do ikin=1,nkina
       write(*,3300) ikin,awa(ikin),xmc(ikin)
      end do

      call carlos(cross,ndim,ncall,cswwa,sdwwa)

      neffww=icall

      write(*,4000)
      write(*,2989)

      write(6,2987) cswwa,sdwwa,neffww

      write(*,4000)
      write(*,3030) dble(ieff)/dble(icall)

      if (idis == 1) then

       do j=1,ndist

       write(*,4000)
        write(6,3000) j
       write(*,4000)

        arg=xmin(j)

        do n=1,nbs(j)

         if (ncs(n,j) > 0 .and. dx(n,j) > zero) then
          val=di(n,j)/dx(n,j)
         else
          val=zero
         end if

         if (j < 3) then
          write(6,1010) sqrt(xdi(n-1,j)),val
          write(6,1010) sqrt(xdi(n,j)),val
         else
          write(6,1010) xdi(n-1,j),val
          write(6,1010) xdi(n,j),val
         end if
        end do

       end do

      write(*,4000)
      write(*,5080)

      write(*,5085) cswwa,neffww

      do j=1,ndist
       write(*,5090) j,sum(di(:,j)),sum(ncs(:,j))
      end do

      end if ! idis



      else

      cswwa=0.d0
      sdwwa=0.d0

      end if ! iwwa


      if (itopa == 1) then

       ndim=2
       ncall=ncalla
       iscan=1

       if (ecm > 2.d0*mt) then
        write(*,4000)
        write(6,2035)
 2035  format('# Approximated cross section e+ e- -> t t~ -> 6f:')

        write(*,4000)


        icall=0

        call carlos(crosstopa,ndim,ncall,csta,sdta)
        neffa=icall
       else
        csta=0.d0
        sdta=0.d0
       end if

      write(*,2989)

      write(6,2987) csta,sdta,neffa

      end if




      acs0(ie)=cs0
      asd0(ie)=sd0
      cstta(ie)=csta
      sdtta(ie)=sdta
      awwa(ie)=cswwa
      asda(ie)=sdwwa

      write(*,4000)

      write(6,2200)
 2200 format('# Born cross sections (fb):')
      write(*,4000)
      write(6,2202)
 2202 format('#    ecm         cs        sd        cstta  ',
     &         '   sda      csbbww     sdbbww')
      do j=1,ie
       write(6,1013) aecm(j),acs0(j),asd0(j),cstta(j),sdtta(j),
     &               awwa(j),asda(j)
      end do
 1013 format(e11.4,e12.5,e9.2,e12.5,e9.2,e12.5,e9.2)


      end do   ! ie

      stop
      end


