      subroutine eeff1(p1,p2,p3,p4,ps1,ps2,ps3,ps4,mat2)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours of
c
c              e+(p1) + e-(p2) -> f(p3) + f~(p4)
c
c Input: pi(0:3), i=1,...,6 - external particle momenta,
c        psi(0:4), i=1,...,6 - spherical components of the external 
c                               particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 3 November, 2000; modified 6 July, 2001
c----------------------------------------------------------------------c

      use mathprms 
      use inprms
      use parproc
      use drivec

      implicit none

      real(8) :: mat2,ebeam=zero
      real(8), dimension(0:3) :: p1,p2,p3,p4
      real(8), dimension(0:4) :: ps1,ps2,ps3,ps4
      integer :: i1,i2,i3,i4,i

      complex(8), dimension(2,2) :: xp1,xp2,xp3,xp4,
     &                              yp1,yp2,yp3,yp4

      complex(8), dimension(4,2) :: u1
      complex(8), dimension(4,2) :: u2
      complex(8), dimension(4,2) :: u3
      complex(8), dimension(4,2) :: u4

      complex(8), dimension(0:3,2,2) :: ea12,ez12
      complex(8), dimension(2,2,2,2) :: xa12,xz12,
     &                                  ya12,yz12

      complex(8), dimension(2,2,2,2) :: amp

      save ebeam,u1,u2,xp1,yp1,xp2,yp2,ea12,xa12,ya12,ez12,xz12,yz12

      if (ebeam /= p1(0)) then

       call spinornew(p1,ps1,m1,-1,1,2,u1)
       call spinornew(p2,ps2,m2,1,0,2,u2)

       call spinr(p1,xp1,yp1)
       call spinr(p2,xp2,yp2)

c Vector boson couplings sandwiched between spinors:

       call fvfa(u1,2,-p1,gll(1),gll(2),zero,ma2,zero,u2,2,-p2,
     &                                                 ea12,xa12,ya12)
       call fvfa(u1,2,-p1,zll(1),zll(2),zero,mz2,zero,u2,2,-p2,
     &                                                 ez12,xz12,yz12)

       ebeam=p1(0)

      end if

c External particle momenta in the spinor representation:


      call spinr(p3,xp3,yp3)
      call spinr(p4,xp4,yp4)

c Spinors of the external fermions:

      call spinornew(p3,ps3,mt,1,1,2,u3)
      call spinornew(p4,ps4,mt,-1,0,2,u4)

c Amplitudes:

      call fef(u3,guu(1)*xa12+zuu(1)*xz12,guu(2)*ya12+zuu(2)*yz12,
     &         onec,onec,u4,amp,2,4,2)


c The spin averaged squared matrix element:

      mat2=zero

      do i1=1,2
      do i2=1,2
      do i3=1,2
      do i4=1,2

       mat2=mat2+abs(amp(i3,i1,i2,i4))**2

      end do
      end do
      end do
      end do

      mat2=colfac2*mat2

      end 



