      subroutine eett6f(nwp,nwm,p1,p2,p3,p4,p5,p6,p7,p8,
     &                  ps1,ps2,ps3,ps4,ps5,ps6,ps7,ps8,mat2)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours of
c
c e+(p1) + e-(p2) --> b(p3) + fu(p4) + fd~(p5) + b~(p6) + fd(p7) + fu~(p8)
c
c
c Input: nwp,nwm = 1/2 if intermidiate W+, W- decays into leptons/hadrons,
c        pi(0:3) - external particle momenta,
c        psi(0:4) - spherical components of the external particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c July-August, 2001; modified 5 September, 2001; 28 March, 2002
c----------------------------------------------------------------------c

      use mathprms 
      use inprms
      use parproc
      use drivec
ckk>
c      use polariz
ckk<
      implicit none

      integer :: nwp,nwm

      real(8), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p8
      real(8), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7,ps8

      real(8) :: ebeam=zero,mat2

      integer :: i,i1,i2,i3,i4,i5,i6,i7,i8,i45,i78,i36

      complex(8), dimension(4) :: xp1,xp2,xp3,xp4,xp5,xp6,xp7,xp8,
     &                            yp1,yp2,yp3,yp4,yp5,yp6,yp7,yp8

      complex(8), dimension(4,2) :: u1,u2,u3,u5,u6,u7
      complex(8), dimension(4,nwp) :: u4
      complex(8), dimension(4,nwm) :: u8


      complex(8), dimension(4,2,2) :: ea36,ez36,
     &                                xa36,ya36,xz36,yz36,
     &                                ea12,xa12,ya12,ez12,xz12,yz12
      complex(8), dimension(2,2) :: h12,h36

      complex(8), dimension(4,nwp,2) :: ew45,xw45,yw45
      complex(8), dimension(4,2,nwm) :: ew78,xw78,yw78
      complex(8), dimension(4,2,nwm,nwp,2) :: eaww,ezww,
     &                                        xaww,yaww,xzww,yzww
      complex(8), dimension(4,2,nwp,2,2) :: e1ww2,x1ww2,y1ww2,
     &                                      e3ww6,x3ww6,y3ww6

      complex(8), dimension(4,2,nwm,2,2) :: e7836,x7836,y7836,
     &                                      e7812,x7812,y7812
      complex(8), dimension(4,nwp,2,2,2) :: e4536,x4536,y4536,
     &                                      e4512,x4512,y4512

      complex(8), dimension(4,2,nwm,2) :: u782,u786
      complex(8), dimension(4,2,nwp,2) :: u145,u345
      complex(8), dimension(4,2,2,2) :: u136,u312,u126
      complex(8), dimension(4,2,nwp,2,2,nwm) :: u14578
      complex(8), dimension(4,2,2,nwm,nwp,2) :: u1ww

      complex(8), dimension(nwp,2,2,nwm) :: h4578

      complex(8) :: mat,scc

      complex(8), dimension(2,2*nwp,2,2,2*nwm,2) :: amp1
      complex(8), dimension(2,2*nwm,2*nwp,2,2,2) :: amp2
      complex(8), dimension(2,2,2*nwp,2,2*nwm,2) :: amp3,amp16
      complex(8), dimension(2,2*nwp,2*nwm,2,2,2) :: amp4
      complex(8), dimension(2,2*nwp,2*nwm,2,2,2) :: amp5
      complex(8), dimension(2,2,2,2*nwp,2*nwm,2) :: amp6
      complex(8), dimension(2,2,2,2*nwm,2*nwp,2) :: amp7
      complex(8), dimension(2,2*nwp,2,2,2*nwm,2) :: amp8
      complex(8), dimension(2,2*nwp,2,2,2*nwm,2) :: amp9
      complex(8), dimension(2*nwm,2,2,2*nwp,2,2) :: amp10
      complex(8), dimension(2*nwp,2,2,2*nwm,2,2) :: amp11
      complex(8), dimension(2*nwp,2*nwm,2,2,2,2) :: amp12,amp13
      complex(8), dimension(2,2,2,2*nwm,2*nwp,2) :: amp14
      complex(8), dimension(2,2,2,2*nwp,2*nwm,2) :: amp15
      complex(8), dimension(2,2*nwm,2*nwp,2,2,2) :: amp17
      complex(8), dimension(2,2*nwp,2,2,2*nwm,2) :: amp18
      complex(8), dimension(2,2*nwp,2*nwm,2,2,2) :: amp19,amp20

      complex(8), dimension(4,2,2,2) :: u362,u365
      complex(8), dimension(4,nwp,2,nwm) :: u478
      complex(8), dimension(4,2,nwm,2) :: u785
      complex(8), dimension(4,nwp,2,2) :: u412,u436
      complex(8), dimension(4,2,2,2) :: u125

      complex(8), dimension(4,2,2,2,2) :: e1a362,x1a362,y1a362,
     &                                    e1z362,x1z362,y1z362,
     &                                    e136a2,x136a2,y136a2,
     &                                    e136z2,x136z2,y136z2
      complex(8), dimension(4,2,2,nwm,2) :: e1782,x1782,y1782,
     &                                      e3786,x3786,y3786
      complex(8), dimension(4,nwp,2,nwm,2) :: e478a5,x478a5,y478a5,
     &                                        e478z5,x478z5,y478z5,
     &                                        e4a785,x4a785,y4a785,
     &                                        e4z785,x4z785,y4z785

      complex(8), dimension(nwp,2*nwm,2,2,2,2,2) :: amp21,amp23,
     &                                              amp26,amg26
      complex(8), dimension(nwp,2,2,2,2,2*nwm,2) :: amp22,amp24,
     &                                              amp27,amg27
      complex(8), dimension(nwp,2,2*nwm,2,2,2,2) :: amp25,amg25
      complex(8), dimension(nwp,2,2,2,2*nwm,2,2) :: amp28
      complex(8), dimension(nwp,2*nwm,2,2,2,2,2) :: amp29,amg29
      complex(8), dimension(nwp,2,2,2*nwm,2,2,2) :: amp30,amg30,amp31
      complex(8), dimension(2,nwp,2*nwm,2,2,2,2) :: amp32,amg32,
     &                                              amp33,amg33
      complex(8), dimension(2,2,2,nwp,2*nwm,2,2) :: amp34,amg34,
     &                                              amp35,amg35
      complex(8), dimension(nwp,2,2,2,2*nwm,2,2) :: amp36,amg36
      complex(8), dimension(nwp,2*nwm,2,2,2,2,2) :: amp37,amg37
      complex(8), dimension(nwp,2,2,2,2,2*nwm,2) :: amp38,amg38
      complex(8), dimension(nwp,2,2*nwm,2,2,2,2) :: amp39
      complex(8), dimension(nwp,2,2,2*nwm,2,2,2) :: amp40,amg40,
     &                                              amp41,amg41
      complex(8), dimension(nwp,2*nwm,2,2,2,2,2) :: amp42

      complex(8), dimension(4,2,2,nwm) :: u128
      complex(8), dimension(4,2,2,nwm) :: u368
      complex(8), dimension(4,nwp,2,nwm) :: u458
      complex(8), dimension(4,2,2,2) :: u712
      complex(8), dimension(4,2,nwp,2) :: u745
      complex(8), dimension(4,2,2,2) :: u736
      complex(8), dimension(4,2,nwp,2,nwm) :: e7a458,x7a458,y7a458,
     &                                        e7z458,x7z458,y7z458,
     &                                        e745a8,x745a8,y745a8,
     &                                        e745z8,x745z8,y745z8
      complex(8), dimension(4,2,2,2,nwm) :: e712w8,x712w8,y712w8,
     &                                      e7w128,x7w128,y7w128
      complex(8), dimension(2,2,2,2,2,2*nwp,nwm) :: amp43,amp47
      complex(8), dimension(2,2*nwp,2,2,2,2,nwm) :: amp44
      complex(8), dimension(2,2,2*nwp,2,2,2,nwm) :: amp45,amg45
      complex(8), dimension(2,2,2,2,2*nwp,2,nwm) :: amp46,amg46
      complex(8), dimension(2,2*nwp,2,2,2,2,nwm) :: amp48,amp57,amg57
      complex(8), dimension(2,2*nwp,2,2,2,2,nwm) :: amp49,amg49,
     &                                              amp50,amg50
      complex(8), dimension(2,2,2,2,2,2*nwp,nwm) :: amp51,amg51,
     &                                              amp55,amg55
      complex(8), dimension(2,2,2,nwm,2*nwp,2,2) :: amp52,amp54
      complex(8), dimension(2,2,2,2*nwp,2,2,nwm) :: amp53,amg53,
     &                                              amp56,amg56,
     &                                              amp58,amg58
      complex(8), dimension(2,2,2,2,2*nwp,nwm,2) :: amp59,amg59,
     &                                              amp60,amg60
      complex(8), dimension(2,2,2*nwp,nwm,2,2,2) :: amp61,amg61,
     &                                              amp62,amg62
      complex(8), dimension(2,2*nwp,2,2,2,nwm,2) :: amp63,amp64


      complex(8), dimension(4,nwp,2,2,2) :: e4w125,x4w125,y4w125,
     &                                      e412w5,x412w5,y412w5
      complex(8), dimension(2,4,nwp,2,2,2,nwm) :: amp65,amg65,
     &                                            amp67,amg67
      complex(8), dimension(2,nwp,2,2,2,4,nwm) :: amp66,amg66,
     &                                            amp68,amg68
      complex(8), dimension(nwp,2,2,2,nwm,4,2) :: amp69,amg69,
     &                                            amp71,amg71
      complex(8), dimension(nwp,4,2,2,2,nwm,2) :: amp70,amg70,
     &                                            amp72,amg72

      integer :: ind,ind45(2,2),ind78(2,2),ind36(2,2)

      save ebeam,u1,u2,xp1,yp1,xp2,yp2,ea12,xa12,ya12,ez12,xz12,yz12,
     &     ind45,ind78,ind36

      if (ebeam /= p1(0)) then

       call spinornew(p1,ps1,m1,-1,1,2,u1)
       call spinornew(p2,ps2,m2,1,0,2,u2)

       call spinr(p1,xp1,yp1)
       call spinr(p2,xp2,yp2)

       call fvfa(u1,2,-p1,gll(1),gll(2),zero,ma2,zero,u2,2,-p2,
     &                                                 ea12,xa12,ya12)
       call fvfa(u1,2,-p1,zll(1),zll(2),zero,mz2,zero,u2,2,-p2,
     &                                                 ez12,xz12,yz12)

      ind=0

       do i5=1,2
        do i4=1,nwp
         ind=ind+1
         ind45(i4,i5)=ind 
        end do
       end do

       ind=0

       do i8=1,nwm
        do i7=1,2
         ind=ind+1
         ind78(i7,i8)=ind 
        end do
       end do

       ind=0

       do i6=1,2
        do i3=1,2
         ind=ind+1
         ind36(i3,i6)=ind 
        end do
       end do

       ebeam=p1(0)

      end if

c External particle momenta in the spinor representation:

      call spinr(p3,xp3,yp3)
      call spinr(p4,xp4,yp4)
      call spinr(p5,xp5,yp5)
      call spinr(p6,xp6,yp6)
      call spinr(p7,xp7,yp7)
      call spinr(p8,xp8,yp8)

c Spinors of the external fermions:

      call spinornew(p3,ps3,m3,1,1,2,u3)
      call spinornew(p4,ps4,m4,1,1,nwp,u4)
      call spinornew(p5,ps5,m5,-1,0,2,u5)
      call spinornew(p6,ps6,m6,-1,0,2,u6)
      call spinornew(p7,ps7,m7,1,1,2,u7)
      call spinornew(p8,ps8,m8,-1,0,nwm,u8)

c Polarization vectors of the intermidiate W's:

      call fvfa(u4,nwp,p4,wff(1),wff(2),zero,mw2,zero,u5,2,p5,
     &                                                 ew45,xw45,yw45)

      call fvfa(u7,2,p7,wff(1),wff(2),zero,mw2,zero,u8,nwm,p8,
     &                                                 ew78,xw78,yw78)

c Polarization vectors of the intermidiate W's:

ckk>
c      call recpol(p4,ps4,mw,ewp)
c      call recpol(p7,ps7,mw,ewm)

c      ewpc=cmplx(ewp,zero,8)
c      ewmc=cmplx(ewm,zero,8)

c      do i=1,3
c       call spinc(ewpc(:,i),xwp(:,i),ywp(:,i))
c      end do
c      do i=1,3
c       call spinc(ewmc(:,i),xwm(:,i),ywm(:,i))
c      end do

c      ew45(:,1,1)=ewpc(:,1)
c      ew45(:,1,2)=ewpc(:,2)
c      ew45(:,2,1)=ewpc(:,3)
c      ew45(:,2,2)=ocom

c      xw45(:,1,1)=xwp(:,1)
c      xw45(:,1,2)=xwp(:,2)
c      xw45(:,2,1)=xwp(:,3)
c      xw45(:,2,2)=ocom

c      yw45(:,1,1)=ywp(:,1)
c      yw45(:,1,2)=ywp(:,2)
c      yw45(:,2,1)=ywp(:,3)
c      yw45(:,2,2)=ocom

c      ew78(:,1,1)=ewmc(:,1)
c      ew78(:,1,2)=ewmc(:,2)
c      ew78(:,2,1)=ewmc(:,3)
c      ew78(:,2,2)=ocom

c      xw78(:,1,1)=xwm(:,1)
c      xw78(:,1,2)=xwm(:,2)
c      xw78(:,2,1)=xwm(:,3)
c      xw78(:,2,2)=ocom

c      yw78(:,1,1)=ywm(:,1)
c      yw78(:,1,2)=ywm(:,2)
c      yw78(:,2,1)=ywm(:,3)
c      yw78(:,2,2)=ocom
ckk<

      call uep(u3,xw45,yw45,wff(1),wff(2),
     &         xp3+xp4+xp5,yp3+yp4+yp5,dm3,ocom,u345,2,2*nwp)

      call peu(-xp6-xp7-xp8,-yp6-yp7-yp8,dm6,ocom,
     &         xw78,yw78,wff(1),wff(2),u6,u786,2*nwm,2)

      call fef(u345,guu(1)*xa12+zuu(1)*xz12,guu(2)*ya12+zuu(2)*yz12,
     &         onec,onec,u786,amp18,4*nwp,4,4*nwm)

      if (itt /= 1) then

C Building blocks:

      call uep(u1,xw45,yw45,wff(1),wff(2),
     &        -xp1+xp4+xp5,-yp1+yp4+yp5,dm1,ocom,u145,2,2*nwp)

      call uep(u145,xw78,yw78,wff(1),wff(2),
     &         xp2-xp3-xp6,yp2-yp3-yp6,cm2,ocom,u14578,4*nwp,2*nwm)

      call peu(xp2-xp7-xp8,yp2-yp7-yp8,dm2,ocom,xw78,yw78,wff(1),wff(2),
     &         u2,u782,2*nwm,2)

      call fvfa(u3,2,p3,gdd(1),gdd(2),zero,ma2,zero,u6,2,p6,
     &                                                 ea36,xa36,ya36)
      call fvfa(u3,2,p3,zdd(1),zdd(2),zero,mz2,zero,u6,2,p6,
     &                                                 ez36,xz36,yz36)

ckk>
c      call recpol(p7,ps7,mz,ez)
c      ezc=cmplx(ez,zero,8)

c      do i=1,3
c       call spinc(ezc(:,i),xz(:,i),yz(:,i))
c      end do

c      ez36(:,1,1)=ezc(:,1)
c      ez36(:,1,2)=ezc(:,2)
c      ez36(:,2,1)=ezc(:,3)
c      ez36(:,2,2)=ocom

c      xz36(:,1,1)=xz(:,1)
c      xz36(:,1,2)=xz(:,2)
c      xz36(:,2,1)=xz(:,3)
c      xz36(:,2,2)=ocom

c      yz36(:,1,1)=yz(:,1)
c      yz36(:,1,2)=yz(:,2)
c      yz36(:,2,1)=yz(:,3)
c      yz36(:,2,2)=ocom

c      ea36=ocom
c      xa36=ocom
c      ya36=ocom

ckk<

      if (ihiggs == 1) then
       call fhfb(u3,2,p3,hff(3),mh2,u6,2,p6,h36)
       call eeh(hww,p4+p5+p7+p8,mh2,ew45,2*nwp,ew78,2*nwm,h4578)
      end if

      call eev(gww,ma2,-p7-p8,ew78,2*nwm,-p4-p5,ew45,2*nwp,
     &                                               eaww,xaww,yaww)
      call eev(zww,mz2,-p7-p8,ew78,2*nwm,-p4-p5,ew45,2*nwp,
     &                                               ezww,xzww,yzww)

      call eve(onec,mw2,-p7-p8,ew78,2*nwm,-p3-p6,gww*ea36+zww*ez36,4,
     &                                ihiggs,hww,h36,e7836,x7836,y7836)

      call eve(onec,mw2,-p7-p8,ew78,2*nwm,p1+p2,gww*ea12+zww*ez12,4,
     &                                 0,hww,h12,e7812,x7812,y7812)

      call vee(onec,mw2,-p4-p5,ew45,2*nwp,-p3-p6,gww*ea36+zww*ez36,4,
     &                                 ihiggs,hww,h36,e4536,x4536,y4536)

      call vee(onec,mw2,-p4-p5,ew45,2*nwp,p1+p2,gww*ea12+zww*ez12,4,
     &                                 0,hww,h12,e4512,x4512,y4512)

      call uep(u1,gll(1)*xaww+zll(1)*xzww,gll(2)*yaww+zll(2)*yzww,
     &         onec,onec,xp2-xp3-xp6,yp2-yp3-yp6,cm1,ocom,
     &         u1ww,2,4*nwm*nwp)

      call uep(u1,gll(1)*xa36+zll(1)*xz36,gll(2)*ya36+zll(2)*yz36,
     &         onec,onec,-xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &         u136,2,4)

      call uep(u3,gdd(1)*xa12+zdd(1)*xz12,gdd(2)*ya12+zdd(2)*yz12,
     &         onec,onec,xp3-xp1-xp2,yp3-yp1-yp2,cm3,ocom,
     &         u312,2,4)

      call fvfa(u145,4*nwp,-p1+p4+p5,wff(1),wff(2),zero,mw2,zero,
     &                                      u2,2,-p2,e1ww2,x1ww2,y1ww2)

      call fvfa(u345,4*nwp,p3+p4+p5,wff(1),wff(2),zero,mw2,zero,u6,2,p6,
     &                                                e3ww6,x3ww6,y3ww6)

      call peu(-xp6+xp1+xp2,-yp6+xp1+xp2,cm6,ocom,
     &         gdd(1)*xa12+zdd(1)*xz12,gdd(2)*ya12+zdd(2)*yz12,
     &         onec,onec,u6,u126,4,2)

c Amplitudes:

      call fef(u145,xz36,yz36,znn(1),znn(2),u782,amp1,4*nwp,4,4*nwm)

      call fef(u1ww,gll(1)*xa36+zll(1)*xz36,gll(2)*ya36+zll(2)*yz36,
     &         onec,onec,u2,amp2,8*nwm*nwp,4,2)

      call fef(u3,x1ww2,y1ww2,wff(1),wff(2),u786,amp3,2,8*nwp,4*nwm)

      call fef(u14578,gll(1)*xa36+zll(1)*xz36,gll(2)*ya36+zll(2)*yz36,
     &         onec,onec,u2,amp4,8*nwp*nwm,4,2)

      call fef(u145,x7836,y7836,wff(1),wff(2),u2,amp5,4*nwp,nwm*8,2)

      call fef(u136,xw45,yw45,wff(1),wff(2),u782,amp6,8,2*nwp,4*nwm)

      call fef(u136,gll(1)*xaww+zll(1)*xzww,gll(2)*yaww+zll(2)*yzww,
     &         onec,onec,u2,amp7,8,4*nwm*nwp,2)

      call fef(u1,x4536,y4536,wff(1),wff(2),u782,amp8,2,nwp*8,4*nwm)

      call fef(u3,x4512,y4512,wff(1),wff(2),u786,amp9,2,nwp*8,4*nwm)

      call eee(-p1-p2+p7+p8,e7812,-p4-p5,ew45,-p3-p6,gww*ea36+zww*ez36,
     &         amp10,nwm*8,2*nwp,4)

      call eee(-p1-p2+p4+p5,e4512,-p7-p8,ew78,-p3-p6,gww*ea36+zww*ez36,
     &         amp11,nwp*8,2*nwm,4)

      call eeee(ew45,ew78,ggww*ea12+gzww*ez12,ea36,
     &                                       amp12,2*nwp,2*nwm,4,4)
      call eeee(ew45,ew78,gzww*ea12+zzww*ez12,ez36,
     &                                       amp13,2*nwp,2*nwm,4,4)

      call fef(u312,gdd(1)*xaww+zdd(1)*xzww,gdd(2)*yaww+zdd(2)*yzww,
     &         onec,onec,u6,amp14,8,4*nwm*nwp,2)

      call fef(u312,xw45,yw45,wff(1),wff(2),u786,amp15,8,2*nwp,4*nwm)

      call fef(u1,x3ww6,y3ww6,wff(1),wff(2),u782,amp16,2,8*nwp,4*nwm)

      call fef(u3,gdd(1)*xaww+zdd(1)*xzww,gdd(2)*yaww+zdd(2)*yzww,
     &         onec,onec,u126,amp17,2,4*nwm*nwp,8)

      call fef(u345,xw78,yw78,wff(1),wff(2),u126,amp19,4*nwp,2*nwm,8)

      call fef(u345,x7812,y7812,wff(1),wff(2),u6,amp20,4*nwp,nwm*8,2)


      if (iww /= 1) then

      call peu(xp2-xp3-xp6,yp2-yp3-yp6,cm2,ocom,
     &         gll(1)*xa36+zll(1)*xz36,gll(2)*ya36+zll(2)*yz36,
     &         onec,onec,u2,u362,4,2)

      call fvfa(u1,2,-p1,gll(1),gll(2),zero,ma2,zero,u362,8,-p2+p3+p6,
     &                                           e1a362,x1a362,y1a362)
      call fvfa(u1,2,-p1,zll(1),zll(2),zero,mz2,zero,u362,8,-p2+p3+p6,
     &                                           e1z362,x1z362,y1z362)
      call uep(u4,xw78,yw78,wff(1),wff(2),
     &         xp4+xp7+xp8,yp4+yp7+yp8,dm4,ocom,u478,nwp,2*nwm)

      call peu(-xp5-xp7-xp8,-yp5-yp7-yp8,dm5,ocom,
     &         xw78,yw78,wff(1),wff(2),u5,u785,2*nwm,2)

      call fef(u478,ga5(1)*x1a362+gz5(1)*x1z362,
     &              ga5(2)*y1a362+gz5(2)*y1z362,onec,onec,u5,
     &                                           amp21,2*nwp*nwm,16,2)

      call fef(u4,ga4(1)*x1a362+gz4(1)*x1z362,
     &            ga4(2)*y1a362+gz4(2)*y1z362,onec,onec,u785,
     &                                           amp22,nwp,16,4*nwm)

      call fvfa(u136,8,-p1+p3+p6,gll(1),gll(2),zero,ma2,zero,u2,2,-p2,
     &                                           e136a2,x136a2,y136a2)
      call fvfa(u136,8,-p1+p3+p6,zll(1),zll(2),zero,mz2,zero,u2,2,-p2,
     &                                           e136z2,x136z2,y136z2)

      call fef(u478,ga5(1)*x136a2+gz5(1)*x136z2,
     &              ga5(2)*y136a2+gz5(2)*y136z2,onec,onec,u5,
     &                                           amp23,2*nwp*nwm,16,2)

      call fef(u4,ga4(1)*x136a2+gz4(1)*x136z2,
     &            ga4(2)*y136a2+gz4(2)*y136z2,onec,onec,u785,
     &                                           amp24,nwp,16,4*nwm)

      call fvfa(u1,2,-p1,wff(1),wff(2),zero,mw2,zero,
     &                       u782,4*nwm,-p2+p7+p8,e1782,x1782,y1782)

      call peu(-xp5-xp3-xp6,-yp5-yp3-yp6,cm5,ocom,
     &         ga5(1)*xa36+gz5(1)*xz36,ga5(2)*ya36+gz5(2)*yz36,
     &         onec,onec,u5,u365,4,2)

      call fef(u4,x1782,y1782,wff(1),wff(2),u365,amp25,nwp,nwm*8,8)

      call fef(u478,ga5(1)*xa12+gz5(1)*xz12,
     &              ga5(2)*ya12+gz5(2)*yz12,onec,onec,u365,
     &                                           amp26,2*nwp*nwm,4,8)

      call uep(u4,ga4(1)*xa12+gz4(1)*xz12,ga4(2)*ya12+gz4(2)*yz12,
     &         onec,onec,xp4-xp1-xp2,yp4-yp1-yp2,cm4,ocom,
     &         u412,nwp,4)

      call fef(u412,ga4(1)*xa36+gz4(1)*xz36,
     &            ga4(2)*ya36+gz4(2)*yz36,onec,onec,u785,
     &                                           amp27,4*nwp,4,4*nwm)

      call fvfa(u3,2,p3,wff(1),wff(2),zero,mw2,zero,
     &                       u786,4*nwm,p6+p7+p8,e3786,x3786,y3786)

      call fef(u412,x3786,y3786,wff(1),wff(2),u5,amp28,4*nwp,8*nwm,2)

      call fef(u4,x7812,y7812,wff(1),wff(2),u365,amp29,nwp,nwm*8,8)

      call fef(u412,xw78,yw78,wff(1),wff(2),u365,amp30,4*nwp,2*nwm,8)

      call fef(u412,x7836,y7836,wff(1),wff(2),u5,amp31,4*nwp,8*nwm,2)

      call fvfa(u478,2*nwp*nwm,p4+p7+p8,ga5(1),ga5(2),zero,ma2,zero,
     &          u5,2,p5,e478a5,x478a5,y478a5)

      call fvfa(u478,2*nwp*nwm,p4+p7+p8,gz5(1),gz5(2),zero,mz2,zero,
     &          u5,2,p5,e478z5,x478z5,y478z5)

      call fvfa(u4,nwp,p4,ga4(1),ga4(2),zero,ma2,zero,
     &          u785,4*nwm,p5+p7+p8,e4a785,x4a785,y4a785)

      call fvfa(u4,nwp,p4,gz4(1),gz4(2),zero,mz2,zero,
     &          u785,4*nwm,p5+p7+p8,e4z785,x4z785,y4z785)

      call fef(u3,ga3(1)*x478a5+gz3(1)*x478z5,
     &            ga3(2)*y478a5+gz3(2)*y478z5,onec,onec,u126,
     &                                           amp32,2,4*nwp*nwm,8)

      call fef(u3,ga3(1)*x4a785+gz3(1)*x4z785,
     &            ga3(2)*y4a785+gz3(2)*y4z785,onec,onec,u126,
     &                                           amp33,2,4*nwp*nwm,8)

      call fef(u312,ga3(1)*x478a5+gz3(1)*x478z5,
     &              ga3(2)*y478a5+gz3(2)*y478z5,onec,onec,u6,
     &                                           amp34,8,4*nwp*nwm,2)

      call fef(u312,ga3(1)*x4a785+gz3(1)*x4z785,
     &              ga3(2)*y4a785+gz3(2)*y4z785,onec,onec,u6,
     &                                           amp35,8,4*nwp*nwm,2)

      call uep(u4,ga4(1)*xa36+gz4(1)*xz36,ga4(2)*ya36+gz4(2)*yz36,
     &         onec,onec,xp4+xp3+xp6,yp4+yp3+yp6,cm4,ocom,
     &         u436,nwp,4)

      call fef(u436,x1782,y1782,wff(1),wff(2),u5,amp36,4*nwp,8*nwm,2)

      call peu(-xp5+xp1+xp2,-yp5+xp1+xp2,cm5,ocom,
     &         ga5(1)*xa12+gz5(1)*xz12,ga5(2)*ya12+gz5(2)*yz12,
     &         onec,onec,u5,u125,4,2)

      call fef(u478,ga5(1)*xa36+gz5(1)*xz36,
     &            ga5(2)*ya36+gz5(2)*yz36,onec,onec,u125,
     &                                           amp37,2*nwp*nwm,4,8)

      call fef(u436,ga4(1)*xa12+gz4(1)*xz12,
     &            ga4(2)*ya12+gz4(2)*yz12,onec,onec,u785,
     &                                           amp38,4*nwp,4,4*nwm)

      call fef(u4,x3786,y3786,wff(1),wff(2),u125,amp39,nwp,8*nwm,8)

      call fef(u436,xw78,yw78,wff(1),wff(2),u125,amp40,4*nwp,2*nwm,8)

      call fef(u436,x7812,y7812,wff(1),wff(2),u5,amp41,4*nwp,8*nwm,2)

      call fef(u4,x7836,y7836,wff(1),wff(2),u125,amp42,nwp,8*nwm,8)




      call peu(-xp8-xp4-xp5,-yp8-yp4-yp5,dm8,ocom,
     &         xw45,yw45,wff(1),wff(2),u8,u458,2*nwp,nwm)
      call fef(u7,ga7(1)*x1a362+gz7(1)*x1z362,
     &            ga7(2)*y1a362+gz7(2)*y1z362,onec,onec,u458,
     &                                           amp43,2,16,2*nwp*nwm)

      call uep(u7,xw45,yw45,wff(1),wff(2),
     &         xp4+xp5+xp7,yp4+yp5+yp7,dm7,ocom,u745,2,2*nwp)
      call fef(u745,ga8(1)*x1a362+gz8(1)*x1z362,
     &              ga8(2)*y1a362+gz8(2)*y1z362,onec,onec,u8,
     &                                           amp44,4*nwp,16,nwm)

      call peu(-xp8-xp3-xp6,-yp8-yp3-yp6,cm8,ocom,
     &         ga8(1)*xa36+gz8(1)*xz36,ga8(2)*ya36+gz8(2)*yz36,
     &         onec,onec,u8,u368,4,nwm)
      call fef(u7,x1ww2,y1ww2,wff(1),wff(2),u368,amp45,2,8*nwp,4*nwm)

      call uep(u7,ga7(1)*xa36+gz7(1)*xz36,ga7(2)*ya36+gz7(2)*yz36,
     &         onec,onec,xp7+xp3+xp6,yp7+yp3+yp6,cm7,ocom,
     &         u736,2,4)
      call fef(u736,x1ww2,y1ww2,wff(1),wff(2),u8,amp46,8,8*nwp,nwm)

      call fef(u7,ga7(1)*x136a2+gz7(1)*x136z2,
     &            ga7(2)*y136a2+gz7(2)*y136z2,onec,onec,u458,
     &                                           amp47,2,16,2*nwp*nwm)

      call fef(u745,ga8(1)*x136a2+gz8(1)*x136z2,
     &              ga8(2)*y136a2+gz8(2)*y136z2,onec,onec,u8,
     &                                           amp48,4*nwp,16,nwm)

      call fvfa(u7,2,p7,ga7(1),ga7(2),zero,ma2,zero,
     &          u458,2*nwm*nwp,p4+p5+p8,e7a458,x7a458,y7a458)
      call fvfa(u7,2,p7,gz7(1),gz7(2),zero,mz2,zero,
     &          u458,2*nwm*nwp,p4+p5+p8,e7z458,x7z458,y7z458)

      call fvfa(u745,4*nwp,p7+p4+p5,ga8(1),ga8(2),zero,ma2,zero,
     &          u8,nwm,p8,e745a8,x745a8,y745a8)
      call fvfa(u745,4*nwp,p7+p4+p5,gz8(1),gz8(2),zero,mz2,zero,
     &          u8,nwm,p8,e745z8,x745z8,y745z8)

      call peu(-xp8+xp1+xp2,-yp8+yp1+yp2,cm8,ocom,
     &         ga8(1)*xa12+gz8(1)*xz12,ga8(2)*ya12+gz8(2)*yz12,
     &         onec,onec,u8,u128,4,nwm)
      call fef(u745,ga8(1)*xa36+gz8(1)*xz36,
     &              ga8(2)*ya36+gz8(2)*yz36,onec,onec,u128,
     &                                           amp49,4*nwp,4,4*nwm)

      call fef(u745,ga8(1)*xa12+gz8(1)*xz12,
     &              ga8(2)*ya12+gz8(2)*yz12,onec,onec,u368,
     &                                           amp50,4*nwp,4,4*nwm)

      call uep(u7,ga7(1)*xa12+gz7(1)*xz12,ga7(2)*ya12+gz7(2)*yz12,
     &         onec,onec,xp7-xp1-xp2,yp7-yp1-yp2,cm7,ocom,
     &         u712,2,4)
      call fef(u712,ga7(1)*xa36+gz7(1)*xz36,
     &              ga7(2)*ya36+gz7(2)*yz36,onec,onec,u458,
     &                                           amp51,8,4,2*nwp*nwm)

      call fvfa(u712,8,p7-p1-p2,wff(1),wff(2),zero,mw2,zero,
     &          u8,nwm,p8,e712w8,x712w8,y712w8)
      call eee(-p1-p2+p7+p8,e712w8,-p4-p5,ew45,-p3-p6,gww*ea36+zww*ez36,
     &         amp52,nwm*8,2*nwp,4)

      call fef(u712,xw45,yw45,wff(1),wff(2),u368,amp53,8,2*nwp,4*nwm)

      call fvfa(u7,2,p7,wff(1),wff(2),zero,mw2,zero,
     &          u128,4*nwm,p8-p1-p2,e7w128,x7w128,y7w128)
      call eee(-p1-p2+p7+p8,e7w128,-p4-p5,ew45,-p3-p6,gww*ea36+zww*ez36,
     &         amp54,nwm*8,2*nwp,4)

      call fef(u736,ga7(1)*xa12+gz7(1)*xz12,
     &              ga7(2)*ya12+gz7(2)*yz12,onec,onec,u458,
     &                                           amp55,8,4,2*nwp*nwm)

      call fef(u736,xw45,yw45,wff(1),wff(2),u128,amp56,8,2*nwp,4*nwm)

      call fef(u7,x4512,y4512,wff(1),wff(2),u368,amp57,2,8*nwp,4*nwm)

      call fef(u736,x4512,y4512,wff(1),wff(2),u8,amp58,8,8*nwp,nwm)

      call fef(u312,ga3(1)*x7a458+gz3(1)*x7z458,
     &              ga3(2)*y7a458+gz3(2)*y7z458,onec,onec,u6,
     &                                           amp59,8,4*nwp*nwm,2)

      call fef(u312,ga3(1)*x745a8+gz3(1)*x745z8,
     &              ga3(2)*y745a8+gz3(2)*y745z8,onec,onec,u6,
     &                                           amp60,8,4*nwp*nwm,2)

      call fef(u3,ga3(1)*x7a458+gz3(1)*x7z458,
     &            ga3(2)*y7a458+gz3(2)*y7z458,onec,onec,u126,
     &                                           amp61,2,4*nwp*nwm,8)

      call fef(u3,ga3(1)*x745a8+gz3(1)*x745z8,
     &            ga3(2)*y745a8+gz3(2)*y745z8,onec,onec,u126,
     &                                           amp62,2,4*nwp*nwm,8)

      call fef(u345,x712w8,y712w8,wff(1),wff(2),u6,amp63,4*nwp,8*nwm,2)

      call fef(u345,x7w128,y7w128,wff(1),wff(2),u6,amp64,4*nwp,8*nwm,2)


      call fvfa(u4,nwp,p4,wff(1),wff(2),zero,mw2,zero,
     &          u125,8,p5-p1-p2,e4w125,x4w125,y4w125)
      call fef(u736,x4w125,y4w125,wff(1),wff(2),u8,amp65,8,8*nwp,nwm)

      call fef(u7,x4w125,y4w125,wff(1),wff(2),u368,amp66,2,8*nwp,4*nwm)

      call fvfa(u412,4*nwp,p4-p1-p2,wff(1),wff(2),zero,mw2,zero,
     &          u5,2,p5,e412w5,x412w5,y412w5)
      call fef(u736,x412w5,y412w5,wff(1),wff(2),u8,amp67,8,8*nwp,nwm)

      call fef(u7,x412w5,y412w5,wff(1),wff(2),u368,amp68,2,8*nwp,4*nwm)

      call fef(u4,x712w8,y712w8,wff(1),wff(2),u365,amp69,nwp,8*nwm,8)

      call fef(u436,x712w8,y712w8,wff(1),wff(2),u5,amp70,4*nwp,8*nwm,2)

      call fef(u4,x7w128,y7w128,wff(1),wff(2),u365,amp71,nwp,8*nwm,8)

      call fef(u436,x7w128,y7w128,wff(1),wff(2),u5,amp72,4*nwp,8*nwm,2)


      if (iew /= 1) then

      if (nwp == 2) then

      call peu(-xp5-xp3-xp6,-yp5-yp3-yp6,cm5,ocom,
     &         xa36,ya36,gg3(1),gg3(2),u5,u365,4,2)

      call fef(u4,x1782,y1782,wff(1),wff(2),u365,amg25,nwp,nwm*8,8)

      call fef(u478,ga5(1)*xa12+gz5(1)*xz12,
     &              ga5(2)*ya12+gz5(2)*yz12,onec,onec,u365,
     &                                           amg26,2*nwp*nwm,4,8)

      call fef(u412,xa36,ya36,gg3(1),gg3(2),u785,amg27,4*nwp,4,4*nwm)

      call fef(u4,x7812,y7812,wff(1),wff(2),u365,amg29,nwp,nwm*8,8)

      call fef(u412,xw78,yw78,wff(1),wff(2),u365,amg30,4*nwp,2*nwm,8)

      call fef(u3,x478a5,y478a5,gg5(1),gg5(2),u126,
     &                                           amg32,2,4*nwp*nwm,8)

      call fef(u3,x4a785,y4a785,gg4(1),gg4(2),u126,
     &                                           amg33,2,4*nwp*nwm,8)

      call fef(u312,x478a5,y478a5,gg5(1),gg5(2),u6,
     &                                           amg34,8,4*nwp*nwm,2)

      call fef(u312,x4a785,y4a785,gg4(1),gg4(2),u6,
     &                                           amg35,8,4*nwp*nwm,2)

      call uep(u4,xa36,ya36,gg3(1),gg3(2),
     &        xp4+xp3+xp6,yp4+yp3+yp6,cm4,ocom,u436,nwp,4)

      call fef(u436,x1782,y1782,wff(1),wff(2),u5,amg36,4*nwp,8*nwm,2)

      call fef(u478,xa36,ya36,gg3(1),gg3(2),u125,amg37,2*nwp*nwm,4,8)

      call fef(u436,ga4(1)*xa12+gz4(1)*xz12,
     &            ga4(2)*ya12+gz4(2)*yz12,onec,onec,u785,
     &                                           amg38,4*nwp,4,4*nwm)

      call fef(u436,xw78,yw78,wff(1),wff(2),u125,amg40,4*nwp,2*nwm,8)

      call fef(u436,x7812,y7812,wff(1),wff(2),u5,amg41,4*nwp,8*nwm,2)

      call fef(u4,x712w8,y712w8,wff(1),wff(2),u365,amg69,nwp,8*nwm,8)

      call fef(u436,x712w8,y712w8,wff(1),wff(2),u5,amg70,4*nwp,8*nwm,2)

      call fef(u4,x7w128,y7w128,wff(1),wff(2),u365,amg71,nwp,8*nwm,8)

      call fef(u436,x7w128,y7w128,wff(1),wff(2),u5,amg72,4*nwp,8*nwm,2)

      end if ! nwp

      if (nwm == 2) then

      call peu(-xp8-xp3-xp6,-yp8-yp3-yp6,cm8,ocom,
     &         xa36,ya36,gg3(1),gg3(2),u8,u368,4,nwm)
      call fef(u7,x1ww2,y1ww2,wff(1),wff(2),u368,amg45,2,8*nwp,4*nwm)

      call uep(u7,xa36,ya36,gg3(1),gg3(2),
     &         xp7+xp3+xp6,yp7+yp3+yp6,cm7,ocom,u736,2,4)
      call fef(u736,x1ww2,y1ww2,wff(1),wff(2),u8,amg46,8,8*nwp,nwm)

      call fef(u745,xa36,ya36,gg3(1),gg3(2),u128,amg49,4*nwp,4,4*nwm)

      call fef(u745,ga8(1)*xa12+gz8(1)*xz12,
     &              ga8(2)*ya12+gz8(2)*yz12,onec,onec,u368,
     &                                           amg50,4*nwp,4,4*nwm)

      call fef(u712,xa36,ya36,gg3(1),gg3(2),u458,amg51,8,4,2*nwp*nwm)

      call fef(u712,xw45,yw45,wff(1),wff(2),u368,amg53,8,2*nwp,4*nwm)

      call fef(u736,ga7(1)*xa12+gz7(1)*xz12,
     &              ga7(2)*ya12+gz7(2)*yz12,onec,onec,u458,
     &                                           amg55,8,4,2*nwp*nwm)

      call fef(u736,xw45,yw45,wff(1),wff(2),u128,amg56,8,2*nwp,4*nwm)

      call fef(u7,x4512,y4512,wff(1),wff(2),u368,amg57,2,8*nwp,4*nwm)

      call fef(u736,x4512,y4512,wff(1),wff(2),u8,amg58,8,8*nwp,nwm)

      call fef(u312,x7a458,y7a458,gg7(1),gg7(2),u6,
     &                                           amg59,8,4*nwp*nwm,2)

      call fef(u312,x745a8,y745a8,gg8(1),gg8(2),u6,
     &                                           amg60,8,4*nwp*nwm,2)

      call fef(u3,x7a458,y7a458,gg7(1),gg7(2),u126,
     &                                           amg61,2,4*nwp*nwm,8)

      call fef(u3,x745a8,y745a8,gg8(1),gg8(2),u126,
     &                                           amg62,2,4*nwp*nwm,8)

      call fvfa(u4,nwp,p4,wff(1),wff(2),zero,mw2,zero,
     &          u125,8,p5-p1-p2,e4w125,x4w125,y4w125)

      call fef(u736,x4w125,y4w125,wff(1),wff(2),u8,amg65,8,8*nwp,nwm)

      call fef(u7,x4w125,y4w125,wff(1),wff(2),u368,amg66,2,8*nwp,4*nwm)

      call fvfa(u412,4*nwp,p4-p1-p2,wff(1),wff(2),zero,mw2,zero,
     &          u5,2,p5,e412w5,x412w5,y412w5)
      call fef(u736,x412w5,y412w5,wff(1),wff(2),u8,amg67,8,8*nwp,nwm)

      call fef(u7,x412w5,y412w5,wff(1),wff(2),u368,amg68,2,8*nwp,4*nwm)

      end if ! nwm

      end if ! iew
      end if ! iww

      end if ! itt


c The spin averaged squared matrix element:

      mat2=zero

      do i1=1,2
      do i2=1,2
      do i3=1,2
      do i4=1,nwp
      do i5=1,2
      do i6=1,2
      do i7=1,2
      do i8=1,nwm

c A sum of the diagrams:

      i36=ind36(i3,i6)
      i45=ind45(i4,i5)
      i78=ind78(i7,i8)

      mat=amp18(i3,i45,i1,i2,i78,i6)                                   ! 56-57

      if (itt /= 1) then

      mat=mat
     &   +amp1(i1,i45,i3,i6,i78,i2)                                   ! 1
     &   -amp2(i1,i78,i45,i3,i6,i2)                                   ! 2-5
     &   +amp3(i3,i1,i45,i2,i78,i6)                                   ! 6
     &   +amp4(i1,i45,i78,i3,i6,i2)                                   ! 7-8
     &   -amp5(i1,i45,i78,i3,i6,i2)                                   ! 9-11
     &   +amp6(i1,i3,i6,i45,i78,i2)                                   ! 12-13
     &   -amp7(i1,i3,i6,i78,i45,i2)                                   ! 14-17
     &   -amp8(i1,i45,i3,i6,i78,i2)                                   ! 18-20
     &   -amp9(i3,i45,i1,i2,i78,i6)                                   ! 23-24
     &  -amp10(i78,i1,i2,i45,i3,i6)                            ! 25-26, 28-29
     &  +amp11(i45,i1,i2,i78,i3,i6)                            ! 31-32, 34-35
     &  -amp12(i45,i78,i1,i2,i3,i6)                                   ! 37,40
     &  -amp13(i45,i78,i1,i2,i3,i6)                                   ! 38-39
     &  -amp14(i3,i1,i2,i78,i45,i6)                            ! 41-42, 44-45
     &  +amp15(i3,i1,i2,i45,i78,i6)                                   ! 47-48
     &  +amp16(i1,i3,i45,i6,i78,i2)                                   ! 49
     &  -amp17(i3,i78,i45,i1,i2,i6)                            ! 50-51, 53-54
     &  +amp19(i3,i45,i78,i1,i2,i6)                                   ! 58-59
     &  -amp20(i3,i45,i78,i1,i2,i6)                                   ! 60-61

      if (ihiggs == 1) then
       mat=mat
     &    +hzz*h4578(i4,i5,i7,i8)*scc(ez12(:,i1,i2),ez36(:,i3,i6))        ! 21
     &    -hzz*h36(i3,i6)*scc(ez12(:,i1,i2),ezww(:,i7,i8,i4,i5))          ! 22
     &    -hww*h36(i3,i6)*scc(e7812(:,i7,i8,i1,i2),ew45(:,i4,i5))    ! 27,30
     &    -hww*h36(i3,i6)*scc(e4512(:,i4,i5,i1,i2),ew78(:,i7,i8))    ! 33,36
     &    +hff(3)*h4578(i4,i5,i7,i8)                                  ! 43,46
     &          *dot_product(conjg(u312(:,i3,i1,i2)),u6(:,i6))
     &    +hff(3)*h4578(i4,i5,i7,i8)                                  ! 52,55
     &          *dot_product(conjg(u3(:,i3)),u126(:,i1,i2,i6))
      end if

      if (iww /= 1) then

      mat=mat
     &   +amp21(i4,i78,i1,i3,i6,i2,i5)
     &   +amp22(i4,i1,i3,i6,i2,i78,i5)
     &   +amp23(i4,i78,i1,i3,i6,i2,i5)
     &   +amp24(i4,i1,i3,i6,i2,i78,i5)
     &   +amp25(i4,i1,i78,i2,i3,i6,i5)
     &   +amp26(i4,i78,i1,i2,i3,i6,i5)
     &   +amp27(i4,i1,i2,i3,i6,i78,i5)
     &   +amp28(i4,i1,i2,i3,i78,i6,i5)
     &   -amp29(i4,i78,i1,i2,i3,i6,i5)
     &   +amp30(i4,i1,i2,i78,i3,i6,i5)
     &   -amp31(i4,i1,i2,i78,i3,i6,i5)
      mat=mat
     &   +amp32(i3,i4,i78,i5,i1,i2,i6)
     &   +amp33(i3,i4,i78,i5,i1,i2,i6)
     &   +amp34(i3,i1,i2,i4,i78,i5,i6)
     &   +amp35(i3,i1,i2,i4,i78,i5,i6)
     &   +amp36(i4,i3,i6,i1,i78,i2,i5)
     &   +amp37(i4,i78,i3,i6,i1,i2,i5)
     &   +amp38(i4,i3,i6,i1,i2,i78,i5)
     &   +amp39(i4,i3,i78,i6,i1,i2,i5)
     &   +amp40(i4,i3,i6,i78,i1,i2,i5)
     &   -amp41(i4,i3,i6,i78,i1,i2,i5)
     &   -amp42(i4,i78,i3,i6,i1,i2,i5)

       mat=mat
     &   +amp43(i7,i1,i3,i6,i2,i45,i8)
     &   +amp44(i7,i45,i1,i3,i6,i2,i8)
     &   +amp45(i7,i1,i45,i2,i3,i6,i8)
     &   +amp46(i7,i3,i6,i1,i45,i2,i8)
     &   +amp47(i7,i1,i3,i6,i2,i45,i8)
     &   +amp48(i7,i45,i1,i3,i6,i2,i8)
     &   +amp49(i7,i45,i3,i6,i1,i2,i8)
     &   +amp50(i7,i45,i1,i2,i3,i6,i8)
     &   +amp51(i7,i1,i2,i3,i6,i45,i8)
     &   +amp52(i7,i1,i2,i8,i45,i3,i6)
     &   +amp53(i7,i1,i2,i45,i3,i6,i8)
       mat=mat
     &   +amp54(i7,i1,i2,i8,i45,i3,i6)
     &   +amp55(i7,i3,i6,i1,i2,i45,i8)
     &   +amp56(i7,i3,i6,i45,i1,i2,i8)
     &   -amp57(i7,i45,i1,i2,i3,i6,i8)
     &   -amp58(i7,i3,i6,i45,i1,i2,i8)
     &   +amp59(i3,i1,i2,i7,i45,i8,i6)
     &   +amp60(i3,i1,i2,i7,i45,i8,i6)
     &   +amp61(i3,i7,i45,i8,i1,i2,i6)
     &   +amp62(i3,i7,i45,i8,i1,i2,i6)
     &   +amp63(i3,i45,i7,i1,i2,i8,i6)
     &   +amp64(i3,i45,i7,i1,i2,i8,i6)


      if (ihiggs == 1) then
       mat=mat
     &    +hzz*h36(i3,i6)*scc(ez12(:,i1,i2),
     &                   e478z5(:,i4,i7,i8,i5)+e4z785(:,i4,i7,i8,i5))
     &    +hzz*h36(i3,i6)*scc(ez12(:,i1,i2),
     &              e7z458(:,i7,i4,i5,i8)+e745z8(:,i7,i4,i5,i8))
     &    +hww*h36(i3,i6)*scc(ew45(:,i4,i5),
     &              e712w8(:,i7,i1,i2,i8)+e7w128(:,i7,i1,i2,i8))
      end if

       mat=mat
     &    +amp65(i7,i36,i4,i1,i2,i5,i8)
     &    +amp66(i7,i4,i1,i2,i5,i36,i8)
     &    +amp67(i7,i36,i4,i1,i2,i5,i8)
     &    +amp68(i7,i4,i1,i2,i5,i36,i8)
     &    +amp69(i4,i7,i1,i2,i8,i36,i5)
     &    +amp70(i4,i36,i7,i1,i2,i8,i5)
     &    +amp71(i4,i7,i1,i2,i8,i36,i5)
     &    +amp72(i4,i36,i7,i1,i2,i8,i5)

      end if ! iww
      end if ! itt

      mat2=mat2+colfac1*abs(mat)**2


      if (itt /= 1) then

      if (iew /= 1 .and. iww /= 1) then

      if (nwp == 2) then

      mat=amg25(i4,i1,i78,i2,i3,i6,i5)
     &   +amg26(i4,i78,i1,i2,i3,i6,i5)
     &   +amg27(i4,i1,i2,i3,i6,i78,i5)
     &   -amg29(i4,i78,i1,i2,i3,i6,i5)
     &   +amg30(i4,i1,i2,i78,i3,i6,i5)
     &   +amg32(i3,i4,i78,i5,i1,i2,i6)
     &   +amg33(i3,i4,i78,i5,i1,i2,i6)
     &   +amg34(i3,i1,i2,i4,i78,i5,i6)
     &   +amg35(i3,i1,i2,i4,i78,i5,i6)
     &   +amg36(i4,i3,i6,i1,i78,i2,i5)
     &   +amg37(i4,i78,i3,i6,i1,i2,i5)
     &   +amg38(i4,i3,i6,i1,i2,i78,i5)
     &   +amg40(i4,i3,i6,i78,i1,i2,i5)
     &   -amg41(i4,i3,i6,i78,i1,i2,i5)
     &   +amg69(i4,i7,i1,i2,i8,i36,i5)
     &   +amg70(i4,i36,i7,i1,i2,i8,i5)
     &   +amg71(i4,i7,i1,i2,i8,i36,i5)
     &   +amg72(i4,i36,i7,i1,i2,i8,i5)

      mat2=mat2+colfac3*abs(mat)**2

      end if ! nwp

      if (nwm == 2) then

      mat=amg45(i7,i1,i45,i2,i3,i6,i8)
     &   +amg46(i7,i3,i6,i1,i45,i2,i8)
     &   +amg49(i7,i45,i3,i6,i1,i2,i8)
     &   +amg50(i7,i45,i1,i2,i3,i6,i8)
     &   +amg51(i7,i1,i2,i3,i6,i45,i8)
     &   +amg53(i7,i1,i2,i45,i3,i6,i8)
     &   +amg55(i7,i3,i6,i1,i2,i45,i8)
     &   +amg56(i7,i3,i6,i45,i1,i2,i8)
     &   -amg57(i7,i45,i1,i2,i3,i6,i8)
     &   -amg58(i7,i3,i6,i45,i1,i2,i8)
     &   +amg59(i3,i1,i2,i7,i45,i8,i6)
     &   +amg60(i3,i1,i2,i7,i45,i8,i6)
     &   +amg61(i3,i7,i45,i8,i1,i2,i6)
     &   +amg62(i3,i7,i45,i8,i1,i2,i6)
     &   +amg65(i7,i36,i4,i1,i2,i5,i8)
     &   +amg66(i7,i4,i1,i2,i5,i36,i8)
     &   +amg67(i7,i36,i4,i1,i2,i5,i8)
     &   +amg68(i7,i4,i1,i2,i5,i36,i8)

      mat2=mat2+colfac3*abs(mat)**2

      end if ! nwm

      end if ! iew and iww

      end if ! itt

      end do
      end do
      end do
      end do
      end do
      end do
      end do
      end do

      mat2=colfac2*mat2

      if (ischeme == 1 .and. ialpw == 1) then
       mat2=calpw*mat2
      end if

      end 



