      subroutine impcuts(p1,ps1,p2,ps2,p3,ps3,p4,ps4,p5,ps5,p6,ps6,
     &                                        p7,ps7,p8,ps8,cross)
c----------------------------------------------------------------------c
c Imposes cuts on the process 
c
c  e+(p1) + e-(p2) --> b(p3) + f1(p4) + f1'~(p5) + b~(p6) + f2(p7) + f2'~(p8)
c
c
c Input: p1,ps1,...,p8,ps8 - four momenta
c
c Output: cross=1.d0/0.d0 if the canonical cuts are satisfied/unsatisfied
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 26 September, 2001;          modified on 18 July, 2002
c----------------------------------------------------------------------c

      use drivec
      use kincuts

      implicit none

      real(8) :: cross,srr
      real(8), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p8
      real(8), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7,ps8

      if (p3(0) < ecutq) then
       cross=0.d0
       return
      else if (p6(0) < ecutq) then
       cross=0.d0
       return
      else if (srr(p3+p6,p3+p6) < mqq**2) then
       cross=0.d0
       return
      else if (abs(ps3(1)) > ctqb) then
       cross=0.d0
       return
      else if (abs(ps6(1)) > ctqb) then
       cross=0.d0
       return
      end if

      if (iwp == 1 .and. iwm == 1) then
       if (p5(0) < ecutl) then
        cross=0.d0
        return
       else if (p7(0) < ecutl) then
        cross=0.d0
        return
       else if (abs(ps5(1)) > ctlb) then
        cross=0.d0
        return
       else if (abs(ps7(1)) > ctlb) then
        cross=0.d0
        return
       else if (dot_product(p5(1:),p3(1:))/ps5(0)/ps3(0) > ctlq) then
        cross=0.d0
        return
       else if (dot_product(p7(1:),p3(1:))/ps7(0)/ps3(0) > ctlq) then
        cross=0.d0
        return
       else if (dot_product(p5(1:),p6(1:))/ps5(0)/ps6(0) > ctlq) then
        cross=0.d0
        return
       else if (dot_product(p7(1:),p6(1:))/ps7(0)/ps6(0) > ctlq) then
        cross=0.d0
        return
       else if (dot_product(p5(1:),p7(1:))/ps5(0)/ps7(0) > ctll) then
        cross=0.d0
        return
       end if
      else if (iwp == 1 .and. iwm == 2) then
       if (p5(0) < ecutl) then
        cross=0.d0
        return
       else if (p7(0) < ecutq) then
        cross=0.d0
        return
       else if (p8(0) < ecutq) then
        cross=0.d0
        return
       else if (srr(p3+p7,p3+p7) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p3+p8,p3+p8) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p6+p7,p6+p7) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p6+p8,p6+p8) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p7+p8,p7+p8) < mqq**2) then
        cross=0.d0
        return
       else if (abs(ps5(1)) > ctlb) then
        cross=0.d0
        return
       else if (abs(ps7(1)) > ctqb) then
        cross=0.d0
        return
       else if (abs(ps8(1)) > ctqb) then
        cross=0.d0
        return
       else if (dot_product(p5(1:),p3(1:))/ps5(0)/ps3(0) > ctlq) then
        cross=0.d0
        return
       else if (dot_product(p5(1:),p6(1:))/ps5(0)/ps6(0) > ctlq) then
        cross=0.d0
        return
       else if (dot_product(p7(1:),p5(1:))/ps7(0)/ps5(0) > ctlq) then
        cross=0.d0
        return
       else if (dot_product(p8(1:),p5(1:))/ps8(0)/ps5(0) > ctlq) then
        cross=0.d0
        return
       end if
      else if (iwp == 2 .and. iwm == 1) then
       if (p4(0) < ecutq) then
        cross=0.d0
        return
       else if (p5(0) < ecutq) then
        cross=0.d0
        return
       else if (p7(0) < ecutl) then
        cross=0.d0
        return
       else if (srr(p3+p4,p3+p4) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p3+p5,p3+p5) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p6+p4,p6+p4) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p6+p5,p6+p5) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p4+p5,p4+p5) < mqq**2) then
        cross=0.d0
        return
       else if (abs(ps4(1)) > ctqb) then
        cross=0.d0
        return
       else if (abs(ps5(1)) > ctqb) then
        cross=0.d0
        return
       else if (abs(ps7(1)) > ctlb) then
        cross=0.d0
        return
       else if (dot_product(p7(1:),p3(1:))/ps7(0)/ps3(0) > ctlq) then
        cross=0.d0
        return
       else if (dot_product(p7(1:),p4(1:))/ps7(0)/ps4(0) > ctlq) then
        cross=0.d0
        return
       else if (dot_product(p7(1:),p6(1:))/ps7(0)/ps6(0) > ctlq) then
        cross=0.d0
        return
       else if (dot_product(p7(1:),p5(1:))/ps7(0)/ps5(0) > ctlq) then
        cross=0.d0
        return
       end if
      else if (iwp == 2 .and. iwm == 2) then
       if (p4(0) < ecutq) then
        cross=0.d0
        return
       else if (p5(0) < ecutq) then
        cross=0.d0
        return
       else if (p7(0) < ecutq) then
        cross=0.d0
        return
       else if (p8(0) < ecutq) then
        cross=0.d0
        return
       else if (srr(p3+p4,p3+p4) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p3+p5,p3+p5) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p3+p7,p3+p7) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p3+p8,p3+p8) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p6+p4,p6+p4) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p6+p5,p6+p5) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p6+p7,p6+p7) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p6+p8,p6+p8) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p4+p5,p4+p5) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p4+p7,p4+p7) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p4+p8,p4+p8) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p5+p7,p5+p7) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p5+p8,p5+p8) < mqq**2) then
        cross=0.d0
        return
       else if (srr(p7+p8,p7+p8) < mqq**2) then
        cross=0.d0
        return
       else if (abs(ps4(1)) > ctqb) then
        cross=0.d0
        return
       else if (abs(ps5(1)) > ctqb) then
        cross=0.d0
        return
       else if (abs(ps7(1)) > ctqb) then
        cross=0.d0
        return
       else if (abs(ps8(1)) > ctqb) then
        cross=0.d0
        return
       end if
      end if ! iwp .and. iwm

      cross=1.d0

      end 


