      subroutine kinee1212(iv1,iv2,iv3,iv4,
     &                   mv1,gamv1,mv2,gamv2,mv3,gamv3,mv4,gamv4,ecm,x,
     &                     p1,ps1,m1,p2,ps2,m2,p3,ps3,m3,p4,ps4,m4,
     &                     p5,ps5,m5,p6,ps6,m6,p7,ps7,m7,p8,ps8,m8,
     &                                                       dlips,flux)
c----------------------------------------------------------------------c
c Returns cartezian and spherical components of four momenta,
c phase space normalization, dlips, and the flux factor for a 2 --> 6
c process in the c.m.s. 
c
c                   p1 + p2 --> p3 + ((p4 + p5) + (p6 + (p7 + p8)))
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Three momentum p3 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c        psi(0) - the length of three momentum pi(1:3),
c        psi(1) - cosinus of the polar angle,
c        psi(2) - sinus of the polar angle,
c        psi(3) - cosinus of the azimuthal angle,
c        psi(4) - sinus of the azimuthal angle.
c
c Input: ivi = 0/1 or 2 if the intermediate state Vi is off/in resonance
c                                                            or massless,
c        ecm - the c.m.s. energy,
c        mvi - intermediate state mass,
c        gamvi - intermediate state width,
c        x(i) - random variables, 0 < x(i) < 1,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 19 September, 2001; modiffied on 
c----------------------------------------------------------------------c

      use mathprms 

      implicit none

      integer :: iv1,iv2,iv3,iv4

      real(8) :: mv1,gamv1,mv2,gamv2,mv3,gamv3,mv4,gamv4,ecm,x(20),
     &           m1,m2,m3,m4,m5,m6,m7,m8,dlips,flux

      real(8), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p8
      real(8), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7,ps8

      real(8), dimension(1:3) :: v,p45

      real(8) :: s45678l,s45678u,s45678,rs45678,ns45678,ats45678,
     &           ds45678,sqs45678,s45l,s45u,s45,ds45,sqs45,
     &           s678l,s678u,s678,ats678,rs678,ns678,ds678,
     &           sqs678,s78l,s78u,s78,sqs78,ds78,
     &           rs45,ns45,ats45,rs78,ns78,ats78,pp,pp3,pp45,pp78,
     &           ct,st,phi,dct,dphi,lamsq,cps,dlips45,
     &           dlips78,g,e,pp4,pp6,pp7,mr2,gvmv,srr

      real(8) :: ecm0=0.d0

      save dct,dphi,cps,ecm0

      if (ecm /= ecm0) then

      pp=sqrt(lamsq(ecm,m1,m2))/(2.d0*ecm)

c Constant parts of the phase space and Jakobian:

      dct=2.d0
      dphi=2.d0*pi
      cps=1.d0/(2.d0*pi)**14

c Flux factor:

      flux=4.d0*ecm*pp

c Initial momenta:

      p1=(/sqrt(m1**2+pp**2),0.d0,0.d0,pp/)
      p2=(/sqrt(m2**2+pp**2),0.d0,0.d0,-pp/)

      ps1=(/pp,1.d0,0.d0,1.d0,0.d0/)
      ps2=(/pp,-1.d0,0.d0,-1.d0,0.d0/)

      ecm0=ecm

      end if

c s45678=(p4+p5+p6+p7+p8)^2:

      s45678l=(m4+m5+m6+m7+m8)**2
      s45678u=(ecm-m3)**2

      if (iv1 == 0) then
       ds45678=s45678u-s45678l
       s45678=ds45678*x(1)+s45678l
      else if (iv1 == 1) then
       gvmv=gamv1*mv1
       mr2=mv1**2
       ats45678=atan((s45678l-mr2)/gvmv)
       ns45678=gvmv/(atan((s45678u-mr2)/gvmv)-ats45678)
       s45678=gvmv*tan(gvmv/ns45678*x(1)+ats45678)+mr2
       ds45678=((s45678-mr2)**2+gvmv**2)/ns45678
       if (ds45678 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv1 == 2) then
       rs45678=s45678u/s45678l
       ns45678=1.d0/log(rs45678)
       s45678=s45678l*rs45678**x(1)
       ds45678=s45678/ns45678
      else
       write(*,*)'iv1 =',iv1,' IS NOT ALLOWED'
       stop
      end if

      sqs45678=sqrt(s45678)

c p3 in the c.m.s.:

      ct=dct*x(2)-1.d0
      st=sqrt(1.d0-ct**2)

      pp3=sqrt(lamsq(ecm,m3,sqs45678))/(2.d0*ecm)

      p3=(/sqrt(m3**2+pp3**2),pp3*st,0.d0,pp3*ct/)

c s45=(p4+p5)^2:

      s45l=(m4+m5)**2
      s45u=(sqs45678-m4-m5)**2

      if (s45u <= s45l) then
       dlips=0.d0
       return
      end if

      if (iv2 == 0) then
       ds45=s45u-s45l
       s45=ds45*x(3)+s45l
      else if (iv2 == 1) then
       gvmv=gamv2*mv2
       mr2=mv2**2
       ats45=atan((s45l-mr2)/gvmv)
       ns45=gvmv/(atan((s45u-mr2)/gvmv)-ats45)
       s45=gvmv*tan(gvmv/ns45*x(3)+ats45)+mr2
       ds45=((s45-mr2)**2+gvmv**2)/ns45
       if (ds45 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv2 == 2) then
       rs45=s45u/s45l
       ns45=1.d0/log(rs45)
       s45=s45l*rs45**x(3)
       ds45=s45/ns45
      else
       write(*,*)'iv2 =',iv2,' IS NOT ALLOWED'
       stop
      end if

      if (s45 < s45l) then
       dlips=0.d0
       return
      else if (s45 > s45u) then
       dlips=0.d0
       return
      end if

      sqs45=sqrt(s45)

c p4 and p5 in their c.m.s.:

      call kinff(sqs45,m4,m5,x(4:5),p4,p5,pp4,dlips45)

      if (dlips45 <= 0.d0) then
       dlips=0.d0
       return
      end if

c s678=(p6+p7+p8)^2:

      s678l=(m6+m7+m8)**2
      s678u=(sqs45678-sqs45)**2

      if (s678u <= s678l) then
       dlips=0.d0
       return
      end if

      if (iv3 == 0) then
       ds678=s678u-s678l
       s678=ds678*x(6)+s678l
      else if (iv3 == 1) then
       gvmv=gamv3*mv3
       mr2=mv3**2
       ats678=atan((s678l-mr2)/gvmv)
       ns678=gvmv/(atan((s678u-mr2)/gvmv)-ats678)
       s678=gvmv*tan(gvmv/ns678*x(6)+ats678)+mr2
       ds678=((s678-mr2)**2+gvmv**2)/ns678
       if (ds678 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv3 == 2) then
       rs678=s678u/s678l
       ns678=1.d0/log(rs678)
       s678=s678l*rs678**x(6)
       ds678=s678/ns678
      else
       write(*,*)'iv3 =',iv3,' IS NOT ALLOWED'
       stop
      end if

      if (s678 < s678l) then
       dlips=0.d0
       return
      else if (s678 > s678u) then
       dlips=0.d0
       return
      end if

      sqs678=sqrt(s678)

c s78=(p7+p8)^2:

      s78l=(m7+m8)**2
      s78u=(sqs678-m6)**2

      if (s78u <= s78l) then
       dlips=0.d0
       return
      end if

      if (iv4 == 0) then
       ds78=s78u-s78l
       s78=ds78*x(7)+s78l
      else if (iv4 == 1) then
       gvmv=gamv4*mv4
       mr2=mv4**2
       ats78=atan((s78l-mr2)/gvmv)
       ns78=gvmv/(atan((s78u-mr2)/gvmv)-ats78)
       s78=gvmv*tan(gvmv/ns78*x(7)+ats78)+mr2
       ds78=((s78-mr2)**2+gvmv**2)/ns78
       if (ds78 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv4 == 2) then
       rs78=s78u/s78l
       ns78=1.d0/log(rs78)
       s78=s78l*rs78**x(7)
       ds78=s78/ns78
      else
       write(*,*)'iv4 =',iv4,' IS NOT ALLOWED'
       stop
      end if

      if (s78 < s78l) then
       dlips=0.d0
       return
      else if (s78 > s78u) then
       dlips=0.d0
       return
      end if

      sqs78=sqrt(s78)

c p6 in the c.m.s. of 6, 7 and 8:

      pp6=sqrt(lamsq(sqs678,m6,sqs78))/(2.d0*sqs678)

      ct=dct*x(8)-1.d0
      st=sqrt(1.d0-ct**2)
      phi=dphi*x(9)

      p6=(/sqrt(m6**2+pp6**2),pp6*st*cos(phi),pp6*st*sin(phi),
     &                                          pp6*ct/)


c p7 and p8 in their c.m.s.:

      call kinff(sqs78,m7,m8,x(10:11),p7,p8,pp7,dlips78)

      if (dlips78 <= 0.d0) then
       dlips=0.d0
       return
      end if

c Direction of p4+p5 in the c.m.s. of 4, 5, 6, 7 and 8:

      ct=dct*x(12)-1.d0
      st=sqrt(1.d0-ct**2)
      phi=dphi*x(13)

      pp45=sqrt(lamsq(sqs45678,sqs45,sqs678))/(2.d0*sqs45678)
      p45=pp45*(/st*cos(phi),st*sin(phi),ct/)

c The phase space normalization:

      dlips=cps*(pi*pp3/ecm)*ds45678*ds678*ds45*ds78*(pi*pp6/sqs678)
     &     *(pi*pp45/sqs45678)*dlips45*dlips78

      if (dlips > zero) then

c Boost p7 and p8 back to the c.m.s. of 6, 7 and 8:

      e=sqrt(s78+pp6**2)
      v=-p6(1:3)/e
      g=e/sqs78

      call boost(g,v,p7)
      call boost(g,v,p8)

c Boost p4, p5, p6, p7 and p8 back to the c.m.s. of 4, 5, 6, 7 and 8:

      e=sqrt(s45+pp45**2)
      v=p45/e
      g=e/sqs45

      call boost(g,v,p4)
      call boost(g,v,p5)

      e=sqrt(s678+pp45**2)
      v=-p45/e
      g=e/sqs678

      call boost(g,v,p6)
      call boost(g,v,p7)
      call boost(g,v,p8)

c Boost p4, p5, p6, p7 and p8 back to the c.m.s.:

      e=sqrt(s45678+pp3**2)
      v=-p3(1:)/e
      g=e/sqs45678

      call boost(g,v,p4)
      call boost(g,v,p5)
      call boost(g,v,p6)
      call boost(g,v,p7)
      call boost(g,v,p8)

c Spherical components of the momenta p3,...,p8:

      call spheric(p3,ps3)
      call spheric(p4,ps4)
      call spheric(p5,ps5)
      call spheric(p6,ps6)
      call spheric(p7,ps7)
      call spheric(p8,ps8)

      end if ! dlips

      end


