      subroutine kinee6f(iv1,iv2,iv3,iv4,
     &                   mv1,gamv1,mv2,gamv2,mv3,gamv3,mv4,gamv4,ecm,x,
     &                     p1,ps1,m1,p2,ps2,m2,p3,ps3,m3,p4,ps4,m4,
     &                     p5,ps5,m5,p6,ps6,m6,p7,ps7,m7,p8,ps8,m8,
     &                                                       dlips,flux)
c----------------------------------------------------------------------c
c Returns cartezian and spherical components of four momenta,
c phase space normalization, dlips, and the flux factor for a 2 --> 6
c process in the c.m.s. 
c
c                   p1 + p2 --> p3 + p4 + (p5 + p6 + (p7 + p8))
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c The three momentum p3 + p4 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c        psi(0) - the length of three momentum pi(1:3),
c        psi(1) - cosinus of the polar angle,
c        psi(2) - sinus of the polar angle,
c        psi(3) - cosinus of the azimuthal angle,
c        psi(4) - sinus of the azimuthal angle.
c
c Input: ivi = 0/1 or 2 if the intermediate state Vi is off/in resonance
c                                                            or massless,
c        ecm - the c.m.s. energy,
c        mvi - intermediate state mass,
c        gamvi - intermediate state width,
c        x(i) - random variables, 0 < x(i) < 1,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 20 July, 2001; modiffied on 
c----------------------------------------------------------------------c

      use mathprms 

      implicit none

      integer :: iv1,iv2,iv3,iv4

      real(8) :: mv1,gamv1,mv2,gamv2,mv3,gamv3,mv4,gamv4,ecm,x(20),
     &           m1,m2,m3,m4,m5,m6,m7,m8,dlips,flux

      real(8), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p8
      real(8), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7,ps8

      real(8), dimension(1:3) :: v

      real(8) :: s34l,s34u,s34,rs34,ns34,ats34,ds34,sqs34,
     &           s56l,s56u,s56,ds56,sqs56,
     &           s5678l,s5678u,s5678,ats5678,rs5678,ns5678,ds5678,
     &           sqs5678,s78l,s78u,s78,sqs78,ds78,
     &           rs56,ns56,ats56,rs78,ns78,ats78,pp,pp3,pp34,pp78,
     &           ct34,st34,ct78,st78,phi78,dct,dphi,lamsq,cps,dlips56,
     &           dlips78,dlips34,g,e,e34,e78,pp5,pp7,mr2,gvmv,srr

      real(8) :: ecm0=0.d0

      save dct,dphi,cps,ecm0

      if (ecm /= ecm0) then

      pp=sqrt(lamsq(ecm,m1,m2))/(2.d0*ecm)

c Constant parts of the phase space and Jakobian:

      dct=2.d0
      dphi=2.d0*pi
      cps=1.d0/(2.d0*pi)**14

c Flux factor:

      flux=4.d0*ecm*pp

c Initial momenta:

      p1=(/sqrt(m1**2+pp**2),0.d0,0.d0,pp/)
      p2=(/sqrt(m2**2+pp**2),0.d0,0.d0,-pp/)

      ps1=(/pp,1.d0,0.d0,1.d0,0.d0/)
      ps2=(/pp,-1.d0,0.d0,-1.d0,0.d0/)

      ecm0=ecm

      end if

c s34=(p3+p4)^2:

      s34l=(m3+m4)**2
      s34u=(ecm-m5-m6-m7-m8)**2

      if (iv1 == 0) then
       ds34=s34u-s34l
       s34=ds34*x(1)+s34l
      else if (iv1 == 1) then
       gvmv=gamv1*mv1
       mr2=mv1**2
       ats34=atan((s34l-mr2)/gvmv)
       ns34=gvmv/(atan((s34u-mr2)/gvmv)-ats34)
       s34=gvmv*tan(gvmv/ns34*x(1)+ats34)+mr2
       ds34=((s34-mr2)**2+gvmv**2)/ns34
       if (ds34 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv1 == 2) then
       rs34=s34u/s34l
       ns34=1.d0/log(rs34)
       s34=s34l*rs34**x(1)
       ds34=s34/ns34
      else
       write(*,*)'iv1 =',iv1,' IS NOT ALLOWED'
       stop
      end if

      sqs34=sqrt(s34)

c s5678=(p5+p6+p7+p8)^2:

      s5678l=(m5+m6+m7+m8)**2
      s5678u=(ecm-sqs34)**2

      if (s5678u <= s5678l) then
       dlips=0.d0
       return
      end if

      if (iv2 == 0) then
       ds5678=s5678u-s5678l
       s5678=ds5678*x(2)+s5678l
      else if (iv2 == 1) then
       gvmv=gamv2*mv2
       mr2=mv2**2
       ats5678=atan((s5678l-mr2)/gvmv)
       ns5678=gvmv/(atan((s5678u-mr2)/gvmv)-ats5678)
       s5678=gvmv*tan(gvmv/ns5678*x(2)+ats5678)+mr2
       ds5678=((s5678-mr2)**2+gvmv**2)/ns5678
       if (ds5678 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv2 == 2) then
       rs5678=s5678u/s5678l
       ns5678=1.d0/log(rs5678)
       s5678=s5678l*rs5678**x(2)
       ds5678=s5678/ns5678
      else
       write(*,*)'iv2 =',iv2,' IS NOT ALLOWED'
       stop
      end if

      if (s5678 < s5678l) then
       dlips=0.d0
       return
      else if (s5678 > s5678u) then
       dlips=0.d0
       return
      end if

      sqs5678=sqrt(s5678)

c s56=(p5+p6)^2:

      s56l=(m5+m6)**2
      s56u=(sqs5678-m7-m8)**2

      if (s56u <= s56l) then
       dlips=0.d0
       return
      end if

      if (iv3 == 0) then
       ds56=s56u-s56l
       s56=ds56*x(3)+s56l
      else if (iv3 == 1) then
       gvmv=gamv3*mv3
       mr2=mv3**2
       ats56=atan((s56l-mr2)/gvmv)
       ns56=gvmv/(atan((s56u-mr2)/gvmv)-ats56)
       s56=gvmv*tan(gvmv/ns56*x(3)+ats56)+mr2
       ds56=((s56-mr2)**2+gvmv**2)/ns56
       if (ds56 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv3 == 2) then
       rs56=s56u/s56l
       ns56=1.d0/log(rs56)
       s56=s56l*rs56**x(3)
       ds56=s56/ns56
      else
       write(*,*)'iv3 =',iv3,' IS NOT ALLOWED'
       stop
      end if

      if (s56 < s56l) then
       dlips=0.d0
       return
      else if (s56 > s56u) then
       dlips=0.d0
       return
      end if

      sqs56=sqrt(s56)

c s78=(p7+p8)^2:

      s78l=(m7+m8)**2
      s78u=(sqs5678-sqs56)**2

      if (s78u <= s78l) then
       dlips=0.d0
       return
      end if

      if (iv4 == 0) then
       ds78=s78u-s78l
       s78=ds78*x(4)+s78l
      else if (iv4 == 1) then
       gvmv=gamv4*mv4
       mr2=mv4**2
       ats78=atan((s78l-mr2)/gvmv)
       ns78=gvmv/(atan((s78u-mr2)/gvmv)-ats78)
       s78=gvmv*tan(gvmv/ns78*x(4)+ats78)+mr2
       ds78=((s78-mr2)**2+gvmv**2)/ns78
       if (ds78 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv4 == 2) then
       rs78=s78u/s78l
       ns78=1.d0/log(rs78)
       s78=s78l*rs78**x(4)
       ds78=s78/ns78
      else
       write(*,*)'iv4 =',iv4,' IS NOT ALLOWED'
       stop
      end if

      if (s78 < s78l) then
       dlips=0.d0
       return
      else if (s78 > s78u) then
       dlips=0.d0
       return
      end if

      sqs78=sqrt(s78)


c p3 and p4 in their c.m.s.:

      call kinff(sqs34,m3,m4,x(5:6),p3,p4,pp3,dlips34)

      if (dlips34 <= 0.d0) then
       dlips=0.d0
       return
      end if

c p5 and p6 in their c.m.s.:

      call kinff(sqs56,m5,m6,x(7:8),p5,p6,pp5,dlips56)

      if (dlips56 <= 0.d0) then
       dlips=0.d0
       return
      end if

c p7 and p8 in their c.m.s.:

      call kinff(sqs78,m7,m8,x(9:10),p7,p8,pp7,dlips78)

      if (dlips78 <= 0.d0) then
       dlips=0.d0
       return
      end if

c p7+p8 in the c.m.s. of 5, 6, 7 and 8:

      ct78=dct*x(11)-1.d0
      st78=sqrt(1.d0-ct78**2)
      phi78=dphi*x(12)

      pp78=sqrt(lamsq(sqs5678,sqs56,sqs78))/(2.d0*sqs5678)


c p3+p4 in the c.m.s.:

      ct34=dct*x(13)-1.d0
      st34=sqrt(1.d0-ct34**2)

      pp34=sqrt(lamsq(ecm,sqs34,sqs5678))/(2.d0*ecm)

c The phase space normalization:

      dlips=cps*(pi*pp34/ecm)*ds34*ds5678*ds56*ds78*(pi*pp78/sqs5678)
     &     *dlips34*dlips56*dlips78

      if (dlips > zero) then

c Boost p5, p6, p7 and p8 back to their relative c.m.s.:

      e78=sqrt(s78+pp78**2)
      v=pp78/e78*(/st78*cos(phi78),st78*sin(phi78),ct78/)
      g=e78/sqs78

      call boost(g,v,p7)
      call boost(g,v,p8)

      e=sqrt(s56+pp78**2)
      v=-e78/e*v
      g=e/sqs56

      call boost(g,v,p5)
      call boost(g,v,p6)

c Boost p3 and p4 back to the c.m.s.:

      e34=sqrt(s34+pp34**2)
      v=pp34/e34*(/st34,0.d0,ct34/)
      g=e34/sqs34

      call boost(g,v,p3)
      call boost(g,v,p4)

c Boost p5, p6, p7 and p8 back to the c.m.s.:

      e=sqrt(s5678+pp34**2)
      v=-e34/e*v
      g=e/sqs5678

      call boost(g,v,p5)
      call boost(g,v,p6)
      call boost(g,v,p7)
      call boost(g,v,p8)

c Spherical components of the momenta p3,...,p8:

      call spheric(p3,ps3)
      call spheric(p4,ps4)
      call spheric(p5,ps5)
      call spheric(p6,ps6)
      call spheric(p7,ps7)
      call spheric(p8,ps8)

      end if ! dlips

      end


