      subroutine kineetbff(iv1,iv2,ecm,mv1,gamv1,mv2,gamv2,x,
     &                     p1,ps1,m1,p2,ps2,m2,p3,ps3,m3,
     &                     p4,ps4,m4,p5,ps5,m5,p6,ps6,m6,dlips,flux)
c----------------------------------------------------------------------c
c Returns cartezian and spherical components of four momenta,
c phase space normalization, dlips, and the flux factor for a 2 --> 4
c process in the c.m.s. 
c
c                   p1 + p2 --> p3 + (p4 + p5 + p6)
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c The three momentum p3 + p4 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c        psi(0) - the length of three momentum pi(1:3),
c        psi(1) - cosinus of the polar angle,
c        psi(2) - sinus of the polar angle,
c        psi(3) - cosinus of the azimuthal angle,
c        psi(4) - sinus of the azimuthal angle.
c
c Input: iv1 = 0/1 or 2 if the intermediate boson V1 is off/in resonance
c                                                            or massless,
c        iv2 = 0/1 or 2 if the intermediate boson V2 is off/in resonance
c                                                            or massless
c        ecm - the c.m.s. energy,
c        mv1,mv2 - intermediate boson masses,
c        gamv1,gamv2 - intermediate boson widhts,
c        x(7) - random variables, 0 < x(i) < 1,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 28 May, 2001; modiffied on 
c----------------------------------------------------------------------c

      use mathprms 

      implicit none

      integer :: iv1,iv2

      real(8) :: ecm,mv1,gamv1,mv2,gamv2,x(10),m1,m2,m3,m4,m5,m6,
     &           s456,s56,dlips,flux

      real(8), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p56,p456
      real(8), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6
      real(8), dimension(1:3) :: v56,v456

      real(8) :: pp,lamsq,cps,s456min,s456max,ds456,s56min,s56max,
     &           ds56,dlips56,gvmv1,mv12,gvmv2,mv22,
     &           dct3,ct3,st3,pp3,
     &           pp5,m32,pp56,rs56,rs456,ns56,ns456,ats56,ats456,sqs56,
     &           sqs456,pv56,pv456,
     &           g56,g456,e56,e456,dlips456,
     &           p0,pst,srr

      real(8) :: ecm0=0.d0

      save cps,ecm0


      if (ecm /= ecm0) then

      pp=sqrt(lamsq(ecm,m1,m2))/(2.d0*ecm)

c Constant parts of the phase space and Jakobian:

      cps=1.d0/(2.d0*pi)**7/(4.d0*ecm)

c Flux factor:

      flux=4.d0*ecm*pp

c Initial momenta:

      p1=(/sqrt(m1**2+pp**2),0.d0,0.d0,pp/)
      p2=(/sqrt(m2**2+pp**2),0.d0,0.d0,-pp/)

      ps1=(/pp,1.d0,0.d0,1.d0,0.d0/)
      ps2=(/pp,-1.d0,0.d0,-1.d0,0.d0/)

      ecm0=ecm

      end if

      if (iv1 == 1) then
       gvmv1=gamv1*mv1
       mv12=mv1**2
      end if

      if (iv2 == 1) then
       gvmv2=gamv2*mv2
       mv22=mv2**2
      end if

c s456=(p4+p5+p6)^2:

      s456min=(m4+m5+m6)**2
      s456max=(ecm-m3)**2

      if (iv1 == 0) then
       ds456=s456max-s456min
       s456=ds456*x(1)+s456min
      else if (iv1 == 1) then
       ats456=atan((s456min-mv12)/gvmv1)
       ns456=gvmv1/(atan((s456max-mv12)/gvmv1)-ats456)
       s456=gvmv1*tan(gvmv1/ns456*x(1)+ats456)+mv12
       if (s456 < s456min) then
        s456=s456min
       end if
       ds456=((s456-mv12)**2+gvmv1**2)/ns456
      else if (iv1 == 2) then
       rs456=s456max/s456min
       ns456=1.d0/log(rs456)
       s456=s456min*rs456**x(1)
       ds456=s456/ns456
      else
       write(*,*)'iv1 =',iv1,' IS NOT ALLOWED'
       stop
      end if

      sqs456=sqrt(s456)

c p3 in the c.m.s.:

      dct3=2.d0
      ct3=dct3*x(2)-1.d0
      st3=sqrt(1.d0-ct3**2)

      m32=m3**2
      p3(0)=(ecm**2-s456+m32)/(2.d0*ecm)

      pp3=sqrt(p3(0)**2-m32)
      p3(1:)=pp3*(/st3,0.d0,ct3/)

c s56=(p5+p6)^2:

      s56min=(m5+m6)**2
      s56max=(sqs456-m4)**2

      if (iv2 == 0) then
       ds56=s56max-s56min
       s56=ds56*x(3)+s56min
      else if (iv2 == 1) then
       ats56=atan((s56min-mv22)/gvmv2)
       ns56=gvmv2/(atan((s56max-mv22)/gvmv2)-ats56)
       s56=gvmv2*tan(gvmv2/ns56*x(3)+ats56)+mv22
       if (s56 < s56min) then
        s56=s56min
       end if
       ds56=((s56-mv22)**2+gvmv2**2)/ns56
      else if (iv2 == 2) then
       rs56=s56max/s56min
       ns56=1.d0/log(rs56)
       s56=s56min*rs56**x(3)
       ds56=s56/ns56
      else
       write(*,*)'iv2 =',iv2,' IS NOT ALLOWED'
       stop
      end if

      sqs56=sqrt(s56)


c p4 and p56=p5+p6 in the c.m.s. of 4, 5 and 6

      call kinff(sqs456,sqs56,m4,x(4:5),p56,p4,pp56,dlips456)

c p5 and p6 in the c.m.s. of 5 and 6

      call kinff(sqs56,m5,m6,x(6:7),p5,p6,pp5,dlips56)

c The phase space normalization:

      dlips=cps*pp3*ds456*dct3*ds56*dlips456*dlips56

      if (dlips > zero) then

c Velocity of the c.m.s. of particles 5 and 6 in the c.m.s. of particles
c 4, 5 and 6:

      e56=sqrt(s56+pp56**2)
      v56=p56(1:)/e56
      g56=e56/sqs56

c Boost p5 and p6 back to the c.m.s. of particles 4, 5 and 6:

      pv56=dot_product(v56,p5(1:))
      p0=p5(0)
      p5(0)=g56*(p0+pv56)
      p5(1:)=p5(1:)+g56*(g56*pv56/(g56+1.d0)+p0)*v56

      pv56=dot_product(v56,p6(1:))
      p0=p6(0)
      p6(0)=g56*(p0+pv56)
      p6(1:)=p6(1:)+g56*(g56*pv56/(g56+1.d0)+p0)*v56

c Velocity of the c.m.s. of particles 4, 5 and 6 in the c.m.s.:

      e456=sqrt(s456+pp3**2)
      v456=-p3(1:)/e456
      g456=e456/sqs456

c Boost p4, p5 and p6 back to the c.m.s.:

      pv456=dot_product(v456,p4(1:))
      p0=p4(0)
      p4(0)=g456*(p0+pv456)
      p4(1:)=p4(1:)+g456*(g456*pv456/(g456+1.d0)+p0)*v456

      pv456=dot_product(v456,p5(1:))
      p0=p5(0)
      p5(0)=g456*(p0+pv456)
      p5(1:)=p5(1:)+g456*(g456*pv456/(g456+1.d0)+p0)*v456

      pv456=dot_product(v456,p6(1:))
      p0=p6(0)
      p6(0)=g456*(p0+pv456)
      p6(1:)=p6(1:)+g456*(g456*pv456/(g456+1.d0)+p0)*v456


c Spherical components of the momenta p3,...,p6:

        ps3=(/pp3,ct3,st3,1.d0,0.d0/)

        ps4(0)=sqrt(p4(1)**2+p4(2)**2+p4(3)**2)
        ps4(1)=p4(3)/ps4(0)
        if (abs(ps4(1)) < 1.d0) then
           ps4(2)=sqrt(1.d0-ps4(1)**2)
           pst=ps4(0)*ps4(2)
           ps4(3)=p4(1)/pst
           ps4(4)=p4(2)/pst
        else if (ps4(1) >= 1.d0) then
           ps4(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps4(1) <= -1.d0) then
           ps4(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

        ps5(0)=sqrt(p5(1)**2+p5(2)**2+p5(3)**2)
        ps5(1)=p5(3)/ps5(0)
        if (abs(ps5(1)) < 1.d0) then
           ps5(2)=sqrt(1.d0-ps5(1)**2)
           pst=ps5(0)*ps5(2)
           ps5(3)=p5(1)/pst
           ps5(4)=p5(2)/pst
        else if (ps5(1) >= 1.d0) then
           ps5(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps5(1) <= -1.d0) then
           ps5(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

        ps6(0)=sqrt(p6(1)**2+p6(2)**2+p6(3)**2)
        ps6(1)=p6(3)/ps6(0)
        if (abs(ps6(1)) < 1.d0) then
           ps6(2)=sqrt(1.d0-ps6(1)**2)
           pst=ps6(0)*ps6(2)
           ps6(3)=p6(1)/pst
           ps6(4)=p6(2)/pst
        else if (ps6(1) >= 1.d0) then
           ps6(1:)=(/1.d0,0.d0,1.d0,0.d0/)
        else if (ps6(1) <= -1.d0) then
           ps6(1:)=(/-1.d0,0.d0,-1.d0,0.d0/)
        end if

      end if ! dlips

      end


